#include "biclique.h"


void biclique::node_add(vector<set<int> >& DB, node& father, set<int> fi, int item, int fi_index)
{
	map<int, node>::iterator itm = father.child_map.find(item);
	
	if(itm == father.child_map.end())   // if there is not this item in the father node;
	{
		node child_i;                    // construct a new node for this item;
		child_i.fi_index = fi_index;     // index to the corresponding frequent item;
		if(father.fi_index == -1)        // if father == root;
		{
			for(int j = 0; j < DB.size(); j++)
			{
				set<int>::iterator itj;
				itj = DB[j].find(item);
				if(itj != DB[j].end())      //DB[j] contains (*it) item;
				{
					child_i.transaction_pool.insert(j);
				}
			}			
		}else                                // if father != root;
		{
			set<int>::iterator its;
			for(its = father.transaction_pool.begin(); its != father.transaction_pool.end(); its++)
			{
				set<int>::iterator itt;
				itt = DB[(*its)].find(item);
				if(itt != DB[(*its)].end())
				{
					child_i.transaction_pool.insert((*its));
				}
			}
		}
		father.child_map.insert(pair<int, node>(item, child_i));

	}else   // if there is this item in the father node;
	{
		node next_node = (*itm).second;
		set<int>::iterator itfi = fi.find(item);
		itfi++;
		if(itfi == fi.end())
		{
			cerr<<"can not add this frequent item in the tree!"<<endl;
			exit(EXIT_FAILURE);
		}else
		{
			node_add(DB, (*itm).second, fi, (*itfi), fi_index);
		}
	}

}

void biclique::prefix_tree(vector<vector<int> >& fi_DB, vector<set<int> >& DB)
{
	// sort fi_DB;
	vector<vector<int> > local_fi = fi_DB;
	myCompare myCompareObject;
	sort(local_fi.begin(), local_fi.end(), myCompareObject); // sort fi;
	vector<set<int> > fi_DB_set;
	for(int i = 0; i < local_fi.size(); i++)
	{
		set<int> e;
		e.insert(local_fi[i].begin(), local_fi[i].end());  // sort item;
		fi_DB_set.push_back(e); 
	}
	node root;
	root.fi_index = -1;
	for(int i = 0; i < fi_DB_set.size(); i++)
	{
		node_add(DB, root, fi_DB_set[i], (*fi_DB_set[i].begin()), i);
	}
	

	// DO THE APPROXIMATION;
	// travel_tree(root, fi_DB_set, APPR_RATIO, root);
	// swap_transaction_pool(root, fi_DB_set); 
	
	// construct base_biclique;
	base_biclique_find(root, fi_DB_set);
}

void biclique::swap_transaction_pool(node &root, vector<set<int> >& fi_DB_set)
{
#ifdef DEBUG
	if(root.fi_index != -1)
	{
		set<int>::iterator it;
		cout<<"["<<root.fi_index<<"]"<<"The FI is: ";
		for(it = fi_DB_set[root.fi_index].begin(); it != fi_DB_set[root.fi_index].end(); it++)
		{
			cout<<(*it)<<" ";
		}
		cout<<";   The ST is: ";
		for(it = root.transaction_pool.begin(); it != root.transaction_pool.end(); it++)
		{
			cout<<(*it)<<" ";
		}
		cout<<endl;
	}
#endif
	cout<<"The FI INDEX is "<<root.fi_index<<": ";
	cout<<"Before swap, ST and AST are "<<root.transaction_pool.size()<<" and "<<root.appr_transaction_pool.size()<<" ."<<endl;
	if(root.appr_transaction_pool.size() != 0)
	{
		root.transaction_pool.swap(root.appr_transaction_pool);
		root.appr_transaction_pool.clear();
	}
	cout<<"After swap, ST and AST are "<<root.transaction_pool.size()<<" and "<<root.appr_transaction_pool.size()<<" ."<<endl;
	
	
    map<int, node>::iterator it;
	for(it = root.child_map.begin(); it != root.child_map.end(); it++)
	{	
		swap_transaction_pool((*it).second, fi_DB_set);
	}
}

void biclique::travel_tree(node& father, vector<set<int> >& fi_DB_set, double approximation_ratio, node& root)
{
	if(father.fi_index != -1)
	{
		do_approximate(father, fi_DB_set, (int)floor(fi_DB_set[father.fi_index].size() * approximation_ratio), root);
	}
	map<int, node>::iterator it;
	for(it = father.child_map.begin(); it != father.child_map.end(); it++)
	{	
		travel_tree((*it).second, fi_DB_set, approximation_ratio, root);
	}
}

void biclique::do_approximate(node& father, vector<set<int> >& fi_DB_set, int relative_level, node& candidate_node)
{
	// if the itemset size is too small;
	if(relative_level == 0)
	{
		return;
	}
	
	if(candidate_node.fi_index == -1)
	{
		// giving sub_nodes;
		map<int, node>::iterator it_map;
		for(it_map = candidate_node.child_map.begin(); it_map != candidate_node.child_map.end(); it_map++)
		{
			set<int>::iterator it_set = fi_DB_set[father.fi_index].find((*it_map).first);
			if(it_set != fi_DB_set[father.fi_index].end())  // pruning;
			{
				do_approximate(father, fi_DB_set, relative_level, (*it_map).second);
			}
		}
	}else
	{
		// do the approximation;
		int a = fi_DB_set[candidate_node.fi_index].size();
		int b = fi_DB_set[father.fi_index].size();
#ifdef DEBUG
		cout<<"a = "<<a<<endl;
		cout<<"b = "<<b<<endl;
		cout<<"relative_level = "<<relative_level<<endl;
#endif
		if(a == (b - relative_level))
		{	
			vector<int> Union;
			vector<int>::iterator it_set;
#ifdef DEBUG
			cout<<"Cand's size = "<<candidate_node.transaction_pool.size()<<endl;
			cout<<"Father's size = "<<father.appr_transaction_pool.size()<<endl;
#endif
			Union.resize(candidate_node.transaction_pool.size() + father.appr_transaction_pool.size());
			it_set = set_union(candidate_node.transaction_pool.begin(), candidate_node.transaction_pool.end(), 
								father.appr_transaction_pool.begin(), father.appr_transaction_pool.end(), Union.begin());
#ifdef DEBUG
			cout<<"before insert?"<<endl;
			cout<<"father.st.size() =  "<<father.appr_transaction_pool.size()<<endl;
#endif
			father.appr_transaction_pool.insert(Union.begin(), it_set);
#ifdef DEBUG
			cout<<"after insert?"<<endl;
			cout<<"father.st.size() =  "<<father.appr_transaction_pool.size()<<endl;
#endif
			return;
		}else
		{	
			map<int, node>::iterator it_map;
			for(it_map = candidate_node.child_map.begin(); it_map != candidate_node.child_map.end(); it_map++)
			{
				set<int>::iterator it_set = fi_DB_set[father.fi_index].find((*it_map).first);
				if(it_set != fi_DB_set[father.fi_index].end())   // pruning;
				{
					do_approximate(father, fi_DB_set, relative_level, (*it_map).second);
				}
			}
		}
	}
}

void biclique::base_biclique_find(node& father, vector<set<int> >& fi_DB)
{
	if(father.fi_index != -1)
	{
		biclique_info bi;
		//cout<<"fi_index is: "<<father.fi_index<<", whose ST size is: "<<father.transaction_pool.size()<<endl;
		int index = father.fi_index;
		bi.a = fi_DB[index];
		bi.b = father.transaction_pool;
		base_biclique.push_back(bi);
	}
	map<int, node>::iterator it;
	for(it = father.child_map.begin(); it != father.child_map.end(); it++)
	{
		base_biclique_find((*it).second, fi_DB);
	}
}

int biclique::biclique_cover(vector<bitset<BIT_LEN> >& M, map<int, set<int> >& DB_covered, map<int, bitset<BIT_LEN> >& map_FI, vector<int>& A, vector<int>& B, char* mfi_file_addr, map<int, int> mapping, char* constant)
{
	// build a copy of MFI;
	for(int i = 0; i < M.size(); i++)
	{
		maximal_FI.insert(i);
	}
	// cout<<maximal_FI.size()<<endl;
#ifdef DEBUG
	cout<<"Base_biclique contains: "<<endl;
	set<int>::iterator h;
	for(int i = 0; i < base_biclique.size(); i++)
	{
		cout<<"A: ";
		for(h = base_biclique[i].a.begin(); h != base_biclique[i].a.end(); h++)
		{
			cout<<(*h)<<" ";
		}
		cout<<"; B: ";
		for(h = base_biclique[i].b.begin(); h != base_biclique[i].b.end(); h++)
		{
			cout<<(*h)<<" ";
		}
		cout<<endl;
	}
#endif	
	// build base biclique set;
	for(int i = 0; i < base_biclique.size(); i++)
	{
		base_biclique_info bbi;
		bbi.base = base_biclique[i];
		optimal_sub_biclique(bbi, maximal_FI, DB_covered, map_FI, A, B, M, constant);
		bbs.insert(pair<double, base_biclique_info>(bbi.price, bbi));
	}
	base_biclique.clear();	
	
	int l = maximal_FI.size();
	while(maximal_FI.size() != 0)
	{
		// print the information for each loop!
		if(l != maximal_FI.size())
		{
			cout<<"In all the "<<M.size()<<" MFIs, "<<((double)(M.size() - maximal_FI.size())/M.size())*100<<" \% have been covered!"<<endl;
			cout<<"The MFIs still contain ( "<< maximal_FI.size() <<" ) sets!"<<endl;
			cout<<"SRC4-V------------------------------------------------------------------------"<<endl;
			l = maximal_FI.size();
		}
#ifdef DEBUG
		set<int>::iterator it_mfi;
		for(it_mfi = maximal_FI.begin(); it_mfi != maximal_FI.end(); it_mfi++)
		{
			int index = (*it_mfi);
			bitset<BIT_LEN> set_b = M[index];
			cout<<" ";
			for(int i = 0; i < set_b.size(); i++)
			{
				if(set_b.test(i))
				{
					cout<<i<<" ";
				}
			}
			cout<<";";
			cout<<(*it_mfi)<<" ";
		}
		cout<<endl;
#endif
		
		multimap<double, base_biclique_info>::iterator it_bbi;
		set<int>::iterator it_m;
		biclique_info candidate;
		set<int> maximal_FI_candidate;
		int base_biclique_index;

		// one case to break;
		if(bbs.size() == 0 && maximal_FI.size() != 0)
		{
			vector<vector<int> > mfi_uncovered(maximal_FI.size(), vector<int>(0));
			set<int>::iterator it;
			int count = 0;
			for(it = maximal_FI.begin(); it != maximal_FI.end(); it++)
			{
				bitset<BIT_LEN> temp_set = M[(*it)];
				for(int i = 0; i < BIT_LEN; i++)
				{
					if(temp_set.test(i))
					{
						map<int, int>::iterator map_it = mapping.find(i);
						assert(map_it != mapping.end());
						mfi_uncovered[count].push_back((*map_it).second);
					}
				}
				count++;
			}
			
			fileRW mfi_rewrite;
			mfi_rewrite.fileOutput(mfi_uncovered, mfi_file_addr);
			break;
		}

		// get the first element of bbs and update it;
		it_bbi = bbs.begin();   // first element;
		assert(it_bbi != bbs.end());
		base_biclique_info bbi_1 = (*it_bbi).second;
		bbs.erase(it_bbi);
		optimal_sub_biclique(bbi_1, maximal_FI, DB_covered, map_FI, A, B, M, constant);
		it_bbi = bbs.begin();  // second element;
		if(it_bbi != bbs.end())
		{
			base_biclique_info bbi_2 = (*it_bbi).second;
			if(bbi_1.price <= bbi_2.price)
			{
				if(bbi_1.price != 0)
				{
					result_info tr;
					tr.bi = bbi_1.sub;
					tr.m_covered = bbi_1.m_covered;
					BC.push_back(tr);
					for(it_m = bbi_1.m_covered.begin(); it_m != bbi_1.m_covered.end(); it_m++)
					{
						maximal_FI.erase((*it_m));
					}
					update_base_biclique(bbi_1);
					if(bbi_1.base.b.size() != 0)
					{
						bbs.insert(pair<double, base_biclique_info>(bbi_1.price, bbi_1));
					}
				}
			}
			else
			{
				if(bbi_1.price != 0)
				{
					bbs.insert(pair<double, base_biclique_info>(bbi_1.price, bbi_1));
				}
			}
		}
		else
		{
			if(bbi_1.price != 0)
			{
				result_info tr;
				tr.bi = bbi_1.sub;
				tr.m_covered = bbi_1.m_covered;
				BC.push_back(tr);
				for(it_m = bbi_1.m_covered.begin(); it_m != bbi_1.m_covered.end(); it_m++)
				{
					// cout<<(*it_m)<<" ";
					maximal_FI.erase((*it_m));
				}
				// cout<<maximal_FI.size()<<endl;
				update_base_biclique(bbi_1);
				if(bbi_1.base.b.size() != 0)
				{
					bbs.insert(pair<double, base_biclique_info>(bbi_1.price, bbi_1));
				}			
			}
		}
	}
	//exit(0);
	if(maximal_FI.size() == 0)
	{
		return maximal_FI.size();
	}else
	{
		return maximal_FI.size();
	}
}

void biclique::update_base_biclique(base_biclique_info& bbi)
{
	set<int>::iterator it;
	for(it = bbi.sub.b.begin(); it != bbi.sub.b.end(); it++)
	{
		bbi.base.b.erase((*it));
	}
}

void biclique::optimal_sub_biclique(base_biclique_info& base_biclique_i, set<int> maximal_FI, map<int, set<int> >& DB_covered, map<int, bitset<BIT_LEN> >& map_FI, vector<int>& A, vector<int>& B, vector<bitset<BIT_LEN> >& M, char* constant)
{
	double min_price = LDBL_MAX;
	double penalty = atof(constant);
	multimap<int, single_X_info> Universe;
	set<int> maximal_FI_covered; // for return;
	biclique_info temp_candidate; // for return;
	temp_candidate.a = base_biclique_i.base.a;	
	set<int>::iterator it;
#ifdef DEBUG
	cout<<"A: ";
	for(it = base_biclique_i.base.a.begin(); it != base_biclique_i.base.a.end(); it++)
	{
		int index = (*it);
		map<int, bitset<BIT_LEN> >::iterator map_it = map_FI.find(A[index]);
		assert(map_it != map_FI.end());
		bitset<BIT_LEN> set_a = (*map_it).second;
		for(int i = 0; i < set_a.size(); i++)
		{
			if(set_a.test(i))
			{
				cout<<i<<" ";
			}
		}
		cout<<", ";
	}
	cout<<" ";
	cout<<"B: ";
	for(it = base_biclique_i.base.b.begin(); it != base_biclique_i.base.b.end(); it++)
	{
		int index = (*it);
		map<int, bitset<BIT_LEN> >::iterator map_it = map_FI.find(B[index]);
		assert(map_it != map_FI.end());
		bitset<BIT_LEN> set_b = (*map_it).second;
		for(int i = 0; i < set_b.size(); i++)
		{
			if(set_b.test(i))
			{
				cout<<i<<" ";
			}
		}
		cout<<", ";
	}
	cout<<endl;
#endif
	
	//exit(0);
	if(base_biclique_i.base.b.size() == 0 || base_biclique_i.base.a.size() == 0)
	{
		cerr<<"the base biclique is empty!"<<endl;
		base_biclique_i.price = 0;
		return;
	}
	
	for(it = base_biclique_i.base.b.begin(); it != base_biclique_i.base.b.end(); it++)
	{
		single_X_info sxi;
		sxi.index = (*it);
		set<int> covered_set;
		set<int>::iterator set_it;
		for(set_it = temp_candidate.a.begin(); set_it != temp_candidate.a.end(); set_it++)
		{
			map<int, set<int> >::iterator map_it;
			map_it = DB_covered.find((*set_it) * MAX + (*it));
			if (map_it != DB_covered.end())
			{
				covered_set.insert((*map_it).second.begin(), (*map_it).second.end());
			}
		}
		
		/* delete the set covered by maximal_FI_covered*/
		vector<int> covered_set_intersection(covered_set.size());
		vector<int>::iterator v_it_1, v_it_2;
		v_it_1 = set_intersection(covered_set.begin(), covered_set.end(), maximal_FI_covered.begin(), maximal_FI_covered.end(), covered_set_intersection.begin());
		for(v_it_2 = covered_set_intersection.begin(); v_it_2 < v_it_1; v_it_2++)
		{
			covered_set.erase((*v_it_2));
		}
		
		/* delete the set uncovered by maximal_FI*/
		covered_set_intersection.clear();
		covered_set_intersection.resize(covered_set.size());
		v_it_1 = set_intersection(covered_set.begin(), covered_set.end(), maximal_FI.begin(), maximal_FI.end(), covered_set_intersection.begin());
		covered_set.clear();
		for(v_it_2 = covered_set_intersection.begin(); v_it_2 < v_it_1; v_it_2++)
		{
			covered_set.insert((*v_it_2));
		}
		sxi.covered = covered_set;
		
		Universe.insert(pair<int, single_X_info>(covered_set.size(), sxi));
	}
#ifdef DEBUG	
	cout<<"Universe.size = "<<Universe.size()<<endl;
#endif
	while(true)
	{
		multimap<int, single_X_info>::iterator it_m;
		it_m = Universe.end();
		it_m--;
		
		single_X_info x_1 = (*it_m).second;
		Universe.erase(it_m);
		vector<int> covered_set_intersection(x_1.covered.size());
		vector<int>::iterator v_it_1, v_it_2;
		v_it_1 = set_intersection(x_1.covered.begin(), x_1.covered.end(), maximal_FI_covered.begin(), maximal_FI_covered.end(), covered_set_intersection.begin());
		for(v_it_2 = covered_set_intersection.begin(); v_it_2 < v_it_1; v_it_2++)
		{
			x_1.covered.erase((*v_it_2));
		}
		
		
		if(Universe.size() != 0)
		{
			it_m = Universe.end();
			it_m--;
			single_X_info x_2 = (*it_m).second;
			if(x_1.covered.size() >= x_2.covered.size())
			{
				double price;
				if((maximal_FI_covered.size() + x_1.covered.size()) == 0)
				{
					price = 0;
				}else
				{
					price = (double)(temp_candidate.a.size() + temp_candidate.b.size() + penalty + 1) / (maximal_FI_covered.size() + x_1.covered.size());
				}
				
				if(price <= min_price && price != 0)
				{
					temp_candidate.b.insert(x_1.index);
					maximal_FI_covered.insert(x_1.covered.begin(), x_1.covered.end());
					min_price = price;
				}else
				{
					if(price > min_price)
					{
						base_biclique_i.sub = temp_candidate;
						base_biclique_i.m_covered = maximal_FI_covered;
						base_biclique_i.price = min_price;
					}else
					{
						min_price = price;
						base_biclique_i.price = min_price;
					}
					break;
				}
			}else
			{
				if(x_1.covered.size() != 0)
				{
					Universe.insert(pair<int, single_X_info>(x_1.covered.size(), x_1));
				}
			}
		}else
		{
			double price;
			if((maximal_FI_covered.size() + x_1.covered.size()) == 0)
			{
				price = 0;
			}else
			{
				price = (double)(temp_candidate.a.size() + temp_candidate.b.size() + penalty + 1) / (maximal_FI_covered.size() + x_1.covered.size());
			}
				
			if(price <= min_price && price != 0)
			{
				temp_candidate.b.insert(x_1.index);
				maximal_FI_covered.insert(x_1.covered.begin(), x_1.covered.end());
				min_price = price;
				
				base_biclique_i.sub = temp_candidate;
				// cout<<base_biclique_i.sub.b.size()<<endl;
				base_biclique_i.m_covered = maximal_FI_covered;
				base_biclique_i.price = min_price;
				break;
			}else
			{
				if(price > min_price)
				{
					base_biclique_i.sub = temp_candidate;
					base_biclique_i.m_covered= maximal_FI_covered;
					base_biclique_i.price = min_price;
				}else
				{
					min_price = price;
					base_biclique_i.price = min_price;
				}
				break;
			}
		}
	}	
}

vector<result_info>& biclique::get_BC()
{
	return BC;
}

vector<biclique_info>& biclique::get_base_BC()
{
	return base_biclique;
}
