#ifndef _BICLIQUE_H_
#define _BICLIQUE_H_

#include <float.h>
#include <algorithm>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include "tools.h"
#include "data.h"

//#define DEBUG
#define APPR_RATIO 0.5


struct biclique_info
{
	set<int> a;
	set<int> b;
};

struct base_biclique_info
{
	biclique_info base;
	biclique_info sub;
	set<int> m_covered;
	double price;
};

struct single_X_info
{
	int index;
	set<int> covered;
};

struct node
{
	map<int, node> child_map;                     // items of children;
	set<int> transaction_pool;                    // supporting transactions;
	set<int> appr_transaction_pool;               // approximation supporting transactions;
	int fi_index;                                 // corresponding fi index;
};

struct result_info
{
	biclique_info bi;
	set<int> m_covered;
};

struct myCompare
{
	bool operator()(vector<int> a, vector<int> b)
	{
		return (a.size() < b.size() );
	}
};

class biclique
{
private:
	vector<result_info> BC; // the final results!
	multimap<double, base_biclique_info> bbs;   // base biclique sets; 
	vector<biclique_info> base_biclique;
	set<int> maximal_FI;
public:
    void swap_transaction_pool(node &root, vector<set<int> >& fi_DB_set);
	void node_add(vector<set<int> >& DB, node& father, set<int> fi, int item, int fi_index);
	void prefix_tree(vector<vector<int> >& fi_DB, vector<set<int> >& DB);
	void travel_tree(node& father, vector<set<int> >& fi_DB_set, double approximation_ratio, node& root);
	void do_approximate(node& father, vector<set<int> >& fi_DB_set, int relative_level, node& candidate_node);
	void base_biclique_find(node& father, vector<set<int> >& fi_DB);
	int  biclique_cover(vector<bitset<BIT_LEN> >& M, map<int, set<int> >& DB_covered, map<int, bitset<BIT_LEN> >& map_FI, vector<int>& A, vector<int>& B, char* mfi_file_addr, map<int, int> mapping, char* constant);
	void update_base_biclique(base_biclique_info& bbi);
	void optimal_sub_biclique(base_biclique_info& base_biclique_i, set<int> maximal_FI, map<int, set<int> >& DB_covered, map<int, bitset<BIT_LEN> >& FI_map, vector<int>& A, vector<int>& B, vector<bitset<BIT_LEN> >& M, char* constant);
	vector<result_info>& get_BC();
	vector<biclique_info>& get_base_BC();
};




#endif
