#include "data.h"

//--------------------------------------------------------------------
// for class FI;
FI::FI(const vector<vector<int> > fi_mfi_vector, const vector<vector<int> > mfi_vector)
{
	// reduce the length of the bitset;
	reduce(mfi_vector);
	
	// sort and get the largest length of all frequent itemsets;
	map<int, int>::iterator it;
	for(int i = 0; i < fi_mfi_vector.size(); i++)
	{
		bitset<BIT_LEN> temp_item;
		for(int j = 0; j < fi_mfi_vector[i].size(); j++)
		{
			it = o_Mapping_r.find(fi_mfi_vector[i][j]);
			assert(it != o_Mapping_r.end());
			temp_item.set((*it).second);
		} 	
		FI_map.insert(pair<int, bitset<BIT_LEN> >(i, temp_item));
		FI_set.insert(temp_item);
		set<bitset<BIT_LEN> >::iterator it;
	}
	// * add empty frequent itemset *;
	bitset<BIT_LEN> empty;
	int empty_KEY = FI_map.size();
	FI_map.insert(pair<int, bitset<BIT_LEN> >(empty_KEY, empty));
}

FI::~FI()
{
	FI_map.clear();
	FI_set.clear();
}

void FI::reduce(vector<vector<int> > fi_vector)
{
	int count = 0;
	map<int, int>::iterator it;
	for(int i = 0; i < fi_vector.size(); i++)
	{
		for(int j = 0; j < fi_vector[i].size(); j++)
		{
			it = o_Mapping_r.find(fi_vector[i][j]);
			if(it == o_Mapping_r.end())
			{
				// cout<<fi_vector[i][j]<<" <--> "<<count<<endl;
				o_Mapping_r.insert(pair<int, int>(fi_vector[i][j], count));
				r_Mapping_o.insert(pair<int, int>(count, fi_vector[i][j]));
				count++;
			}
		}
	}
}

map<int ,int>& FI::get_rmapo()
{
	return r_Mapping_o;
}

map<int ,int>& FI::get_omapr()
{
	return o_Mapping_r;
}

map<int, bitset<BIT_LEN> >& FI::getFI_map()
{
	return FI_map;
}

set<bitset<BIT_LEN>, fiCompare >& FI::getFI_set()
{
	return FI_set;
}

//------------------------------------------------------------------
// for class itemset_construct

itemset_construct::itemset_construct()
{

}

itemset_construct::~itemset_construct()
{

}

void itemset_construct::construct_A(map<int, bitset<BIT_LEN> > &FI_map)
{
	map<int, bitset<BIT_LEN> >::iterator it;
	for(it = FI_map.begin(); it != FI_map.end(); it++)
	{
		if((*it).second.count() <= SPLIT_NUM)
		{
			A.push_back((*it).first);
		}
	}
}

void itemset_construct::construct_B(map<int, bitset<BIT_LEN> > &FI_map)
{
	map<int, bitset<BIT_LEN> >::iterator it;
	for(it = FI_map.begin(); it != FI_map.end(); it++)
	{
		if((*it).second.count() > SPLIT_NUM)
		{
			B.push_back((*it).first);
		}
	}
	
	it = FI_map.end();
	it--;
	B.push_back((*it).first);
}

void itemset_construct::construct_M(vector<vector<int> > mfi_vector, map<int, int> mapping)
{
	// change mfi to bitset;
	for(int i = 0; i < mfi_vector.size(); i++)
	{
		bitset<BIT_LEN> temp_item;
		for(int j = 0; j < mfi_vector[i].size(); j++)
		{
			map<int ,int>::iterator it = mapping.find(mfi_vector[i][j]);
			assert(it != mapping.end());
			temp_item.set((*it).second);
		}
		M.push_back(temp_item);
	}
}

vector<int>& itemset_construct::get_A()
{
	return A;
}

vector<int>& itemset_construct::get_B()
{
	return B;
}

vector<bitset<BIT_LEN> >& itemset_construct::get_M()
{
	return M;
}

//--------------------------------------------------------------------------
// for class bipartite_graph

bipartite_graph::bipartite_graph()
{
	
}

bipartite_graph::~bipartite_graph()
{

}

vector<vector<int> > bipartite_graph::construct_graph(map<int, bitset<BIT_LEN> >& FI_map, set<bitset<BIT_LEN>, fiCompare >& FI_set, vector<int>& A, vector<int>& B, vector<bitset<BIT_LEN> >& M, char* Option)
{
	vector<vector<int> > file_content(B.size(), vector<int> (0));
	DB.resize(B.size());
	map<int, bitset<BIT_LEN> >::iterator it_A, it_B;
	for(int i = 0; i < B.size(); i++)
	{
		for(int j = 0; j < A.size(); j++)
		{		
			// cout<<"I= "<<i<<"; J= "<<j<<endl;
			it_A = FI_map.find(A[j]);
			assert(it_A != FI_map.end());
			it_B = FI_map.find(B[i]);
			assert(it_B != FI_map.end());
			bitset<BIT_LEN> set_combined;
			set_combined = (*it_A).second | (*it_B).second;
			bool distance_check = false;
			
			for(int k = 0; k < M.size(); k++)
			{
				bitset<BIT_LEN> AND_set = set_combined & M[k];
				
				if(set_combined == M[k])  // for sparce graph;
				{
					file_content[i].push_back(j);
				}
				
				if(!strcmp(Option, "2"))
				{
					if(AND_set == M[k])
					{
						bitset<BIT_LEN> XOR_set = set_combined ^ M[k];
						if(XOR_set.count() <= BIT_DISTANCE)
						{
							distance_check = true;
						}
						if(DB_covered.find(j*MAX + i) != DB_covered.end())
						{
							DB_covered[j*MAX + i].insert(k);
#ifdef DEBUG
							cout<<"DB_KEY: "<<j*MAX + i;
							cout<<"KEY( "<<A[j]<<" )";
							cout<<"A: ";
							map<int, bitset<BIT_LEN> >::iterator map_a = FI_map.find(A[j]);
							assert(map_a != FI_map.end());
							bitset<BIT_LEN> set_a = (*map_a).second;
							cout<<" ";
							for(int l = 0; l != set_a.size(); l++)
							{
								if(set_a.test(l))
									cout<<l<<" ";
							}
							cout<<";  ";

							cout<<"KEY( "<<B[i]<<" )";
							cout<<"B: ";
			                map<int, bitset<BIT_LEN> >::iterator map_b = FI_map.find(B[i]);
							assert(map_b != FI_map.end());
							bitset<BIT_LEN> set_b = (*map_b).second;
							cout<<" ";
							for(int l = 0; l != set_b.size(); l++)
							{
								if(set_b.test(l))
									cout<<l<<" ";
							}
							cout<<";  ";
							
							cout<<"M:";
							bitset<BIT_LEN> set_m = M[k];
							cout<<" ";
							for(int l = 0; l != set_m.size(); l++)
							{
								if(set_m.test(l))
									cout<<l<<" ";
							}
							cout<<";";
							cout<<endl;

#endif
						}else
						{
							set<int> temp_set;
							temp_set.insert(k);
							DB_covered.insert(pair<int,set<int> >(j * MAX + i, temp_set));						
#ifdef DEBUG							
							cout<<"DB_KEY: "<<j*MAX + i;
							cout<<"KEY( "<<A[j]<<" )";
							cout<<"A: ";
							map<int, bitset<BIT_LEN> >::iterator map_a = FI_map.find(A[j]);
							assert(map_a != FI_map.end());
							bitset<BIT_LEN> set_a = (*map_a).second;
							cout<<" ";
							for(int l = 0; l != set_a.size(); l++)
							{
								if(set_a.test(l))
									cout<<l<<" ";
							}
							cout<<";  ";

							cout<<"KEY( "<<B[i]<<" )";
							cout<<"B: ";
			                map<int, bitset<BIT_LEN> >::iterator map_b = FI_map.find(B[i]);
							assert(map_b != FI_map.end());
							bitset<BIT_LEN> set_b = (*map_b).second;
							cout<<" ";
							for(int l = 0; l != set_b.size(); l++)
							{
								if(set_b.test(l))
									cout<<l<<" ";
							}
							cout<<";  ";


							cout<<"M:"<<endl;
							bitset<BIT_LEN> set_m = M[k];
							cout<<" ";
							for(int l = 0; l != set_m.size(); l++)
							{
								if(set_m.test(l))
									cout<<l<<" ";
							}
							cout<<";";
							cout<<endl;
#endif
						}
					}
				}
				
				if(!strcmp(Option, "2") || !strcmp(Option, "1"))
				{
					if(AND_set == set_combined)
					{
						DB[i].insert(j);
						if(AND_set == M[k])
						{
							if(DB_covered.find(j*MAX + i) != DB_covered.end())
							{
								DB_covered[j*MAX + i].insert(k);
#ifdef DEBUG
							cout<<"DB_KEY: "<<j*MAX + i;
							cout<<"KEY( "<<A[j]<<" )";
							cout<<"A: ";
							map<int, bitset<BIT_LEN> >::iterator map_a = FI_map.find(A[j]);
							assert(map_a != FI_map.end());
							bitset<BIT_LEN> set_a = (*map_a).second;
							cout<<" ";
							for(int l = 0; l != set_a.size(); l++)
							{
								if(set_a.test(l))
									cout<<l<<" ";
							}
							cout<<";  ";

							cout<<"KEY( "<<B[i]<<" )";
							cout<<"B: ";
			                map<int, bitset<BIT_LEN> >::iterator map_b = FI_map.find(B[i]);
							assert(map_b != FI_map.end());
							bitset<BIT_LEN> set_b = (*map_b).second;
							cout<<" ";
							for(int l = 0; l != set_b.size(); l++)
							{
								if(set_b.test(l))
									cout<<l<<" ";
							}
							cout<<";  ";
							
							cout<<"M:";
							bitset<BIT_LEN> set_m = M[k];
							cout<<" ";
							for(int l = 0; l != set_m.size(); l++)
							{
								if(set_m.test(l))
									cout<<l<<" ";
							}
							cout<<";";
							cout<<endl;

#endif
							}else
							{
								set<int> temp_set;
								temp_set.insert(k);
								DB_covered.insert(pair<int,set<int> >(j * MAX + i, temp_set));						
#ifdef DEBUG							
							cout<<"DB_KEY: "<<j*MAX + i;
							cout<<"KEY( "<<A[j]<<" )";
							cout<<"A: ";
							map<int, bitset<BIT_LEN> >::iterator map_a = FI_map.find(A[j]);
							assert(map_a != FI_map.end());
							bitset<BIT_LEN> set_a = (*map_a).second;
							cout<<" ";
							for(int l = 0; l != set_a.size(); l++)
							{
								if(set_a.test(l))
									cout<<l<<" ";
							}
							cout<<";  ";

							cout<<"KEY( "<<B[i]<<" )";
							cout<<"B: ";
			                map<int, bitset<BIT_LEN> >::iterator map_b = FI_map.find(B[i]);
							assert(map_b != FI_map.end());
							bitset<BIT_LEN> set_b = (*map_b).second;
							cout<<" ";
							for(int l = 0; l != set_b.size(); l++)
							{
								if(set_b.test(l))
									cout<<l<<" ";
							}
							cout<<";  ";


							cout<<"M:"<<endl;
							bitset<BIT_LEN> set_m = M[k];
							cout<<" ";
							for(int l = 0; l != set_m.size(); l++)
							{
								if(set_m.test(l))
									cout<<l<<" ";
							}
							cout<<";";
							cout<<endl;
#endif
							}
						}
					}
				}
			}
			
			if(!strcmp(Option, "2"))
			{
				if(distance_check == false)
				{
					map<int, set<int> >::iterator it = DB_covered.find(j*MAX + i);
					if(it != DB_covered.end())
					{
						DB_covered.erase(it);
					}
				}else
				{
					DB[i].insert(j);
				}
			}
		}
		if(file_content[i].size() == 0)
		{
			file_content[i].push_back(MAX);
		}
		
		
	}
	return file_content;
}

void bipartite_graph::write_FI_DB(vector<vector<int> > file_content, char* fileAddress)
{
	fileRW file_write;
	file_write.fileOutput(file_content, fileAddress);
}

map<int,set<int> >& bipartite_graph::get_DB_covered()
{
	return DB_covered;
}

vector<set<int> >& bipartite_graph::get_DB()
{
	return DB;
}
