#include "tools.h"
#include "data.h"
#include "biclique.h"
#include <iostream>
#include <iomanip.h>
#include <stdlib.h>
#include <bitset>
#include <time.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "assert.h"

//#define DEBUG

using namespace std;

int main (int argc, char** argv)
{
	if(argc < 7)
	{
		cerr<<"This algorithm's usages are as follows: "<<endl;
		cerr<<"Usage: "<<argv[0]<<" [Input_File_Name]   [Min_Sup_1]   [Min_Sup_2]   [Min_Sup_3]   [First or not]   Option   constant"<<endl;
		cerr<<"                  |                 |             |             |             |              |            |"<<endl;
		cerr<<"                  |                 |             |             |             |              |            |"<<endl;
		cerr<<"Ex:    "<<argv[0]<<"    abcd.dat            0.3           0.3           0.4            Y              1            10 "<<endl;
		exit(-1);
	}
	
	/*---------------------------------------------------------------------------------------------*/
	/*   call the software Mafia to generate the frequent itemsets and maximal frequent itemsets;  */
	int i = 0;

	string mafia_address("../Mafia/src/");
	string dataset_address("../datasets/");

	/* for maximal frequent itemset !*/
	string input_file_name = dataset_address;
	input_file_name += argv[1];
	
	string output_file_name = dataset_address;    // output file;
	output_file_name += "output/mfi_";
	output_file_name += argv[1];

	string command = mafia_address;
	command += "./mafia -mfi ";
	command += argv[2];
	command += " -ascii ";
	command += input_file_name;
	command += " ";
	command += output_file_name;
	cout<<command<<endl;
	char* c_command_1 = new char[command.size() + 1];
	strcpy(c_command_1, command.c_str());
	if(!strcmp(argv[5],"Y"))
	{
		cout<<"read new MFI ........"<<endl;
		i = system(c_command_1);
	}else
	{
		cout<<"read the left MFI ......"<<endl;
	}
	if(i == 127 || i == -1)
	{
		cerr<<"the call to Maifa failed..."<<endl;
		exit(-1);
	}
	cout<<"MFI is generated!"<<endl<<endl;
	/*----------------------------------------------------------------------------------------------*/
	/*          read the closed frequent itemsets and get the frequent itemsets of them            */
	char* c_mfi_addr = new char[output_file_name.size() + 1];
	strcpy(c_mfi_addr, output_file_name.c_str());

	string output_addr = dataset_address;                                  // the addr for FI_MFI;
	output_addr += "output/FI_MFI_";
	output_addr += argv[1];

	/* to deal with the "()" in the data file */
	fileRW mfi_handle;
	vector<vector<int> > mfi = mfi_handle.fileInput(c_mfi_addr); 
	if(mfi.size() == 0)
	{
		cerr<<endl<<"The size of Maximal Frequent Itemsets is ZERO -_-!"<<endl;
		exit(-1);
	}
	mfi_handle.fileOutput(mfi, c_mfi_addr);
	
	command.clear();
	command += mafia_address;
	command += "./mafia -fci ";
	command += argv[3];
	command += " -ascii ";
	command += output_file_name;
	command += " ";
	command += output_addr;
	char* c_command_2 = new char[command.size() + 1];
	strcpy(c_command_2, command.c_str());
	cout<<command<<endl;
	i = system(c_command_2);
	if(i == 127 || i == -1)
	{
		cerr<<"the call to Maifa failed..."<<endl;
		exit(-1);
	}
	cout<<"FI_MFI is generated!"<<endl;
	/*----------------------------------------------------------------------------------------------*/
	/* read files and obtain the original frequent itemsets and maximal frequent itemsets */
	char * c_fi_mfi_addr = new char[output_addr.size() + 1];
	strcpy(c_fi_mfi_addr, output_addr.c_str());
	
	fileRW file;
	vector<vector<int> > FI_MFI = file.fileInput(c_fi_mfi_addr);   // for FI_MFI;
#ifdef DEBUG
	for(int i = 0; i < FI_MFI.size(); i++)
	{
		for(int j = 0; j < FI_MFI[i].size(); j++)
		{
			cout<<FI_MFI[i][j]<<" ";
		}
		cout<<", ";
	}
	cout<<endl;
#endif
	vector<vector<int> > MFI = file.fileInput(c_mfi_addr);      // MFI;
	cout<<endl<<"|MFI| = "<<MFI.size()<<endl;
	cout<<"|FI_MFI| = "<<FI_MFI.size()<<endl;

	/*----------------------------------------------------------------------------------------------*/
	/*                              prepare data for the algorithm!                                 */
	map<int,bitset<BIT_LEN> > mapfi;
	set<bitset<BIT_LEN>, fiCompare> setfi;	
	map<int, int> mapping_0, mapping_1;
	FI fi(FI_MFI, MFI); // FI object;
	mapfi = fi.getFI_map();
	setfi = fi.getFI_set();
	mapping_0 = fi.get_omapr();
	mapping_1 = fi.get_rmapo();
	cout<<"Reduce BIT-LEN to "<<mapping_0.size()<<" !!"<<endl;
	itemset_construct ic;    // itemset_construct object;
	ic.construct_A(mapfi);
	ic.construct_B(mapfi);
	ic.construct_M(MFI, mapping_0);
	vector<int> A,B;
	vector<bitset<BIT_LEN> > M;
	A = ic.get_A();
	B = ic.get_B();
	M = ic.get_M();
	cout<<"A and B are generated ......"<<endl;
	///*----------------------------------------------------------------------------------------------*/
	///* store the maximal fi covered by each pair from A and B */
	vector<vector<int> > dbg;
	vector<set<int> > db;
	map<int, set<int> > dbc;
	bipartite_graph bg;
	cout<<"|A| = "<<A.size()<<endl;
	cout<<"|B| = "<<B.size()<<endl;
	if(A.size() > B.size())
	{
		A.swap(B);
		cout<<"After swap,"<<endl;
		cout<<"|A| = "<<A.size()<<endl;
		cout<<"|B| = "<<B.size()<<endl;
	}
	time_t start, end;
	double dif;
	time(&start);
	cout<<"Starting to construct DB ......"<<endl;
	dbg = bg.construct_graph(mapfi,setfi,A,B,M, argv[6]);
	cout<<"Constructing finished ......"<<endl;
	time(&end);
	dif = difftime(end, start);
	cout<<"Takes "<<dif<<" seconds to construct Bipartite Graph!"<<endl;
	dbc = bg.get_DB_covered();
	db = bg.get_DB();
	///*----------------------------------------------------------------------------------------------*/
	///*      write the file which will be read by apriori and read the fi and transaction files        */
	string addr_dbg = dataset_address;   // for input;
	addr_dbg += "output/DB_";
	addr_dbg += argv[1];
	char* addr_3 = new char[addr_dbg.size() + 1];
	strcpy(addr_3, addr_dbg.c_str());
	bg.write_FI_DB(dbg, addr_3);  

	string apriori_out_addr = dataset_address;   // for output;
	apriori_out_addr += "output/fi_DB_";
	apriori_out_addr += argv[1];
	char* addr_4 = new char[apriori_out_addr.size() + 1];
	strcpy(addr_4, apriori_out_addr.c_str());

	
	double percent_d = atof(argv[4]);    // for commmand;
	percent_d *= 100;
	char percent_c[10];
	sprintf(percent_c, "%f", percent_d);
	string apriori_command("./apriori -ts -s");
	apriori_command += percent_c;
	apriori_command += " ";
	apriori_command += addr_3;
	apriori_command += " ";
	apriori_command += addr_4;
	char* c_apriori_command = new char[apriori_command.size() + 1];
	strcpy(c_apriori_command, apriori_command.c_str());
	cout<<c_apriori_command<<endl;

	system(c_apriori_command);
	///*-----------------------------------------------------------------------------------------*/
	///*                            find the biclique cover ! */
	 ////cout<<"the address is"<<addr_4<<endl;
	vector<vector<int> > fi_DB = file.fileInput(addr_4);
	// check fi_DB for containing MAX;
	vector<int> index_container;
	for(int i = 0; i < fi_DB.size(); i++)
	{
		for(int j = 0; j < fi_DB[i].size(); j++)
		{
			if(fi_DB[i][j] == MAX)
			{
				index_container.push_back(i);
			}
		}
	}
	
	while(index_container.size() != 0)
	{
		fi_DB.erase(fi_DB.begin() + index_container[0]);
		index_container.erase(index_container.begin());
		for(int i = 0; i < index_container.size(); i++)
		{
			index_container[i]--;
		}
	}
	
	if(fi_DB.size() ==0)
	{
		cerr<<endl<<"Frequent Itemsets of Bipartite Graph are EMPTY......"<<endl<<endl;
		
		// if No such directory, create a new one;
		string dir_addr("result/");
		string tmp = argv[1];
		tmp.erase((int)tmp.find_last_of("."));
		dir_addr += tmp;
		dir_addr += "/";
		DIR * dp;
		if((dp = opendir(dir_addr.c_str())) == NULL)
		{
			string command("mkdir -p ");
			command += dir_addr;
			cout<<command<<endl;
			system(command.c_str());
		}
		
		// write the file;
		string connection_file = "result/";
	    connection_file += tmp;
		connection_file += "/";
		connection_file += tmp;
		connection_file += "_covered_ratio.txt";
		cout<<connection_file<<endl;
		fstream connection;
		connection.open(connection_file.c_str(), fstream::out);
		connection<<"0"<<endl;
		connection.close();
		exit(0);
	}
	
	cout<<"|FI of DB| = "<<fi_DB.size()<<endl;
	biclique bcq;
	cout<<"Starting to Construct Prefix_Tree ......"<<endl;
	time(&start);
	bcq.prefix_tree(fi_DB, db);
	time(&end);
	dif = difftime(end, start);
	cout<<"Constructing finished ......"<<endl;
	cout<<"Takes "<<dif<<" seconds to construct Prefix_Tree ......"<<endl;
	vector<biclique_info> bbc = bcq.get_base_BC();
	cout<<"|Base Biclique| = "<<bbc.size()<<endl;
 	time(&start);
	int redo = bcq.biclique_cover(M, dbc, mapfi, A, B, c_mfi_addr, mapping_1, argv[7]);
	time(&end);
	
	
	// to make the "result" directory or not!
	string dir_addr_0("result/");
	DIR * dp_0;
	if((dp_0 = opendir(dir_addr_0.c_str())) == NULL)
	{
		string command("mkdir -p ");
		command += dir_addr_0;
		cout<<command<<endl;
		system(command.c_str());
	}
	
	vector<result_info> result = bcq.get_BC();
	//write the biclique;
	string addr_bc("result/biclique_");
	addr_bc += argv[1];
	char* addr_5 = new char[addr_bc.size() + 1];
	strcpy(addr_5, addr_bc.c_str());
	
	double b_cost = 0;
	double m_cost = 0;
	int b_total_length = 0;
	int m_total_length = 0;
	set<int> distinct_a, distinct_b;
	fstream bc_output;
	bc_output.open(addr_5, fstream::out|fstream::app);
	// if(bc_output.is_open())
	// {
		// cout<<"The biclique file is opened successully!"<<endl;
	// }else
	// {
		// cout<<"Error! Openning the biclique file fails!"<<endl;
	// }
	set<int>::iterator it;
	for(int i = 0; i < result.size(); i++)
	{
		b_cost += result[i].bi.a.size() + result[i].bi.b.size();
		m_cost += result[i].m_covered.size();
		bc_output<<"Biclique [ "<<i<<" ] , Cost[ "<<(result[i].bi.a.size() + result[i].bi.b.size())<<" ],  MFI_covered["<<result[i].m_covered.size()<<"]: "<<endl;
		bc_output<<"A contains: "<<endl;
		for(it= result[i].bi.a.begin(); it != result[i].bi.a.end(); it++)
		{
			int index = (*it);
			distinct_a.insert(index);
			map<int, bitset<BIT_LEN> >::iterator map_a = mapfi.find(A[index]);
			assert(map_a != mapfi.end());
			bitset<BIT_LEN> set_a = (*map_a).second;
			bc_output<<" ";
			for(int i = 0; i < set_a.size(); i++)
			{
				if(set_a.test(i))
				{
					map<int, int>::iterator map_it = mapping_1.find(i);
					assert(map_it != mapping_1.end());
					bc_output<<(*map_it).second<<" ";
					b_total_length++;
				}
			}
			bc_output<<";";
		}
		bc_output<<endl;			

		bc_output<<"B contains: "<<endl;
		for(it= result[i].bi.b.begin(); it != result[i].bi.b.end(); it++)
		{
			int index = (*it);
			distinct_b.insert(index);
			map<int, bitset<BIT_LEN> >::iterator map_b = mapfi.find(B[index]);
			assert(map_b != mapfi.end());
			bitset<BIT_LEN> set_b = (*map_b).second;
			bc_output<<" ";
			for(int i = 0; i < set_b.size(); i++)
			{
				if(set_b.test(i))
				{
					map<int, int>::iterator map_it = mapping_1.find(i);
					assert(map_it != mapping_1.end());
					bc_output<<(*map_it).second<<" ";
					b_total_length++;
				}
			}
			bc_output<<";";
		}
		bc_output<<endl;
		
		bc_output<<"Covered_MFI contains: "<<endl;
		for(it= result[i].m_covered.begin(); it != result[i].m_covered.end(); it++)
		{
			int index = (*it);
			assert(index < M.size());
			bitset<BIT_LEN> set_m = M[index];
			bc_output<<" ";
			for(int i = 0; i <set_m.size(); i++)
			{
				if(set_m.test(i))
				{
					map<int, int>::iterator map_it = mapping_1.find(i);
					assert(map_it != mapping_1.end());
					bc_output<<(*map_it).second<<" ";
					m_total_length++;
				}
			}
			bc_output<<";";
		}
		bc_output<<endl;
	}
	dif = difftime(end, start);
	bc_output<<"PARAMETERS:  ALPHA "<<argv[2]<<"; BETA "<<argv[3]<<"; GAMMA "<<argv[4]<<"; MEHTHOD "<<argv[6]<<"; CONSTANT "<<argv[7]<<". "<<endl;
	bc_output<<"TOTAL MFI :   "<<MFI.size()<<"; COVERED MFI:   "<<m_cost<<";   "<<"TOTAL LENGTH of COVERED MFI:   "<<m_total_length<<". "<<endl;
	bc_output<<"TOTAL BIC :   "<<result.size()<<";   TOTAL COST:   "<<b_cost<<";   "<<"TOTAL LENGTH of ALL SETs:      "<<b_total_length<<". "<<endl;
	bc_output<<"DISTINCT ITEMS in BIC: "<<distinct_a.size() + distinct_b.size()<<". "<<endl;
	bc_output<<dif<<" SECONDs to find the bicliques!"<<endl;
	// cout<<"The Biclique file has been written!";
	bc_output.close();
	
	// cout<<distinct_a.size()<<endl;
	// cout<<distinct_b.size()<<endl;
	// cout<<"Now write the distinct set"<<endl;
	// set<int>::iterator dis_it;
	// for(dis_it= distinct_a.begin(); dis_it != distinct_a.end(); dis_it++)
		// {
			// int index = (*dis_it);
			// map<int, bitset<BIT_LEN> >::iterator map_a = mapfi.find(A[index]);
			// assert(map_a != mapfi.end());
			// bitset<BIT_LEN> set_a = (*map_a).second;
			// cout<<" ";
			// set<int> it;
			// set<int>::iterator iit;
			// for(int i = 0; i <set_a.size(); i++)
			// {
				// if(set_a.test(i))
				// {
					// map<int, int>::iterator map_it = mapping_1.find(i);
					// assert(map_it != mapping_1.end());
					// cout<<(*map_it).second<<" ";
				// }
			// }
			
			// cout<<endl;
		// }
	// for(dis_it= distinct_b.begin(); dis_it != distinct_b.end(); dis_it++)
		// {
			// int index = (*dis_it);
			// map<int, bitset<BIT_LEN> >::iterator map_b = mapfi.find(B[index]);
			// assert(map_b != mapfi.end());
			// bitset<BIT_LEN> set_b = (*map_b).second;
			// cout<<" ";
			// set<int> it;
			// set<int>::iterator iit;
			// for(int i = 0; i <set_b.size(); i++)
			// {
				// if(set_b.test(i))
				// {
					// map<int, int>::iterator map_it = mapping_1.find(i);
					// assert(map_it != mapping_1.end());
					// cout<<(*map_it).second<<" ";
				// }
			// }
			
			// cout<<endl;
		// }
	cout<<endl<<"--------------------------------------------------------------------------------"<<endl;
	cout<<"PARAMETERS:  ALPHA "<<argv[2]<<"; BETA "<<argv[3]<<"; GAMMA "<<argv[4]<<"; MEHTHOD "<<argv[6]<<"; CONSTANT "<<argv[7]<<". "<<endl;
	cout<<"TOTAL MFI : "<<MFI.size()<<"; COVERED MFI: "<<m_cost<<"; "<<"TOTAL LENGTH of COVERED MFI: "<<m_total_length<<". "<<endl;
	cout<<"TOTAL BIC : "<<result.size()<<"; TOTAL COST: "<<b_cost<<"; "<<"TOTAL LENGTH of ALL SET: "<<b_total_length<<". "<<endl;
	cout<<"DISTINCT ITEMS in BIC: "<<distinct_a.size() + distinct_b.size()<<". "<<endl;
	cout<<dif<<" SECONDs to find the bicliques!"<<endl;
	cout<<"--------------------------------------------------------------------------------"<<endl<<endl;;
	
	//exit(0);
	// create the directory to store biclique files;
	string dir_addr("result/");
	string tmp = argv[1];
	tmp.erase((int)tmp.find_last_of("."));
	dir_addr += tmp;
	dir_addr += "/biclique/";
	DIR * dp;
	if((dp = opendir(dir_addr.c_str())) == NULL)
	{
		string command("mkdir -p ");
		command += dir_addr;
		cout<<command<<endl;
		system(command.c_str());
	}
	
	// create the biclique files;
	string bc_file(dir_addr);
	bc_file += argv[2];
	bc_file += ".txt";
	bc_output.open(bc_file.c_str(), fstream::out|fstream::app);
	for(int i = 0; i < result.size(); i++)
	{
		for(it= result[i].bi.a.begin(); it != result[i].bi.a.end(); it++)
		{
			int index = (*it);
			map<int, bitset<BIT_LEN> >::iterator map_a = mapfi.find(A[index]);
			assert(map_a != mapfi.end());
			bitset<BIT_LEN> set_a = (*map_a).second;
			bc_output<<" ";
			for(int i = 0; i < set_a.size(); i++)
			{
				if(set_a.test(i))
				{
					map<int, int>::iterator map_it = mapping_1.find(i);
					assert(map_it != mapping_1.end());
					bc_output<<(*map_it).second<<" ";
				}
			}
			bc_output<<", ";
		}
		bc_output<<": ";			

		for(it= result[i].bi.b.begin(); it != result[i].bi.b.end(); it++)
		{
			int index = (*it);
			map<int, bitset<BIT_LEN> >::iterator map_b = mapfi.find(B[index]);
			assert(map_b != mapfi.end());
			bitset<BIT_LEN> set_b = (*map_b).second;
			bc_output<<" ";
			for(int i = 0; i < set_b.size(); i++)
			{
				if(set_b.test(i))
				{
					map<int, int>::iterator map_it = mapping_1.find(i);
					assert(map_it != mapping_1.end());
					bc_output<<(*map_it).second<<" ";
				}
			}
			bc_output<<", ";
		}
		bc_output<<endl;
	}
	bc_output.close();
	
	int tl = 0;
	
	for(int i = 0; i < MFI.size(); i++)
	{
		tl += MFI[i].size();
	}
	
	string dir_addr0("result/");
	dir_addr0 += tmp;
	dir_addr0 += "/parameter/";
	cout<<"dir_addr0: "<<dir_addr0<<endl;
	if((dp = opendir(dir_addr0.c_str())) == NULL)
	{
		string command("mkdir -p ");
		command += dir_addr0;
		cout<<command<<endl;
		system(command.c_str());
	}
	
	bc_file.clear();
	bc_file += dir_addr0;
	bc_file += argv[2];
	bc_file += ".txt";
	
	
	bc_output.open(bc_file.c_str(), fstream::out|fstream::app);
	bc_output<<"ALPHA:\t"<<argv[2]<<"\tBETA:\t"<<argv[3]<<"\tGAMMA:\t"<<argv[4]<<"\tMEHTHOD:\t"<<argv[6]<<"\tCONSTANT:\t"<<argv[7]<<"\tTOTAL MFI:\t"<<MFI.size()<<"\tTOTAL LENGTH of MFI:\t"<<tl<<endl;
	bc_output.close();
	/*----------------------------------------------------------------------------------------------*/
	
	
	double ratio = ((double)(MFI.size()-redo))/MFI.size();
	int RATIO = (int) floor(ratio * 100);
	string connection_file = "result/";
	connection_file += tmp;
	connection_file += "/";
	connection_file += tmp;
	connection_file += "_covered_ratio.txt";
	cout<<connection_file<<endl;
	fstream connection;
	connection.open(connection_file.c_str(), fstream::out);
	connection<<RATIO<<endl;
	connection.close();
	if(RATIO == 100)
	{
		cout<<"*****************************************************"<<endl<<endl;
		cout<<"100 % MFIs have been covered... :-) "<<endl;
		cout<<endl<<"*****************************************************"<<endl;
		cout<<"PROGRAM FINISHED!"<<endl;
		return 0;
	}else
	{
		cout<<"*****************************************************"<<endl<<endl;
		cout<<"Only "<<RATIO<<" % MFIs have been covered... -_-!"<<endl;
		cout<<endl<<"*****************************************************"<<endl;
		cout<<"PROGRAM FINISHED!"<<endl;
		return 1;
	}
}
