/*

Copyright (c) 2003, Cornell University
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    - Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
    - Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
    - Neither the name of Cornell University nor the names of its
        contributors may be used to endorse or promote products derived from
        this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

*/


/////////////////////////////////////////////////////////////////////
///
/// Tables.h
///
// For gathering the lookup tables in one place.
///
///
/////////////////////////////////////////////////////////////////////

#ifndef TABLES_H

#define TABLES_H

/// BitTable[i] contains a 1 at only position i
const int BitTable[32] = {
                             2147483648UL, 1073741824UL, 536870912UL, 268435456UL,
                             134217728UL, 67108864UL, 33554432UL, 16777216UL,
                             8388608UL, 4194304UL, 2097152UL, 1048576UL,
                             524288UL, 262144UL, 131072UL, 65536UL,
                             32768UL, 16384UL, 8192UL, 4096UL,
                             2048UL, 1024UL, 512UL, 256UL,
                             128UL, 64UL, 32UL, 16UL,
                             8UL, 4UL, 2UL, 1UL
                         };

/// CountTable[i] contains the number of 1's in the byte value of i
const char CountTable[256] = {
                                 0UL, 1UL, 1UL, 2UL, 1UL, 2UL, 2UL, 3UL, 1UL, 2UL, 2UL, 3UL, 2UL, 3UL, 3UL, 4UL, 1UL, 2UL, 2UL, 3UL, 2UL, 3UL, 3UL, 4UL, 2UL, 3UL, 3UL, 4UL, 3UL, 4UL, 4UL, 5UL, 1UL, 2UL, 2UL, 3UL, 2UL, 3UL, 3UL, 4UL, 2UL, 3UL, 3UL, 4UL, 3UL, 4UL, 4UL, 5UL, 2UL, 3UL, 3UL, 4UL, 3UL, 4UL, 4UL, 5UL, 3UL, 4UL, 4UL, 5UL, 4UL, 5UL, 5UL, 6UL, 1UL,
                                 2UL, 2UL, 3UL, 2UL, 3UL, 3UL, 4UL, 2UL, 3UL, 3UL, 4UL, 3UL, 4UL, 4UL, 5UL, 2UL, 3UL, 3UL, 4UL, 3UL, 4UL, 4UL, 5UL, 3UL, 4UL, 4UL, 5UL, 4UL, 5UL, 5UL, 6UL, 2UL, 3UL, 3UL, 4UL, 3UL, 4UL, 4UL, 5UL, 3UL, 4UL, 4UL, 5UL, 4UL, 5UL, 5UL, 6UL, 3UL, 4UL, 4UL, 5UL, 4UL, 5UL, 5UL, 6UL, 4UL, 5UL, 5UL, 6UL, 5UL, 6UL, 6UL, 7UL, 1UL,
                                 2UL, 2UL, 3UL, 2UL, 3UL, 3UL, 4UL, 2UL, 3UL, 3UL, 4UL, 3UL, 4UL, 4UL, 5UL, 2UL, 3UL, 3UL, 4UL, 3UL, 4UL, 4UL, 5UL, 3UL, 4UL, 4UL, 5UL, 4UL, 5UL, 5UL, 6UL, 2UL, 3UL, 3UL, 4UL, 3UL, 4UL, 4UL, 5UL, 3UL, 4UL, 4UL, 5UL, 4UL, 5UL, 5UL, 6UL, 3UL, 4UL, 4UL, 5UL, 4UL, 5UL, 5UL, 6UL, 4UL, 5UL, 5UL, 6UL, 5UL, 6UL, 6UL, 7UL, 2UL,
                                 3UL, 3UL, 4UL, 3UL, 4UL, 4UL, 5UL, 3UL, 4UL, 4UL, 5UL, 4UL, 5UL, 5UL, 6UL, 3UL, 4UL, 4UL, 5UL, 4UL, 5UL, 5UL, 6UL, 4UL, 5UL, 5UL, 6UL, 5UL, 6UL, 6UL, 7UL, 3UL, 4UL, 4UL, 5UL, 4UL, 5UL, 5UL, 6UL, 4UL, 5UL, 5UL, 6UL, 5UL, 6UL, 6UL, 7UL, 4UL, 5UL, 5UL, 6UL, 5UL, 6UL, 6UL, 7UL, 5UL, 6UL, 6UL, 7UL, 6UL, 7UL, 7UL, 8UL
                             };

/////////////////////////////////////////////////////////////////////
/// CompTable is used for compression step.
/// CompTable[i][j] contains the value of (byte value j) COMPRESSED
/// relative to (byte value i) and was already calculated as follows:
///
///   - result = NULL;
///   - for each bit position P with a 1 in (byte value i)
///   - append value of bit position P in (byte value j) to result
///
/////////////////////////////////////////////////////////////////////
const unsigned int CompTable[256][256] = {
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL
            },
            {
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL,
                0UL,
                2147483648UL
            },
            {
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL
            },
            {
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL
            },
            {
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL
            },
            {
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL
            },
            {
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL
            },
            {
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL
            },
            {
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL
            },
            {
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL
            },
            {
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL
            },
            {
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL
            },
            {
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL
            },
            {
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                0UL,
                0UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4160749568UL,
                4227858432UL,
                0UL,
                67108864UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                67108864UL,
                67108864UL,
                134217728UL,
                134217728UL,
                201326592UL,
                201326592UL,
                268435456UL,
                268435456UL,
                335544320UL,
                335544320UL,
                402653184UL,
                402653184UL,
                469762048UL,
                469762048UL,
                536870912UL,
                536870912UL,
                603979776UL,
                603979776UL,
                671088640UL,
                671088640UL,
                738197504UL,
                738197504UL,
                805306368UL,
                805306368UL,
                872415232UL,
                872415232UL,
                939524096UL,
                939524096UL,
                1006632960UL,
                1006632960UL,
                1073741824UL,
                1073741824UL,
                1140850688UL,
                1140850688UL,
                1207959552UL,
                1207959552UL,
                1275068416UL,
                1275068416UL,
                1342177280UL,
                1342177280UL,
                1409286144UL,
                1409286144UL,
                1476395008UL,
                1476395008UL,
                1543503872UL,
                1543503872UL,
                1610612736UL,
                1610612736UL,
                1677721600UL,
                1677721600UL,
                1744830464UL,
                1744830464UL,
                1811939328UL,
                1811939328UL,
                1879048192UL,
                1879048192UL,
                1946157056UL,
                1946157056UL,
                2013265920UL,
                2013265920UL,
                2080374784UL,
                2080374784UL,
                2147483648UL,
                2147483648UL,
                2214592512UL,
                2214592512UL,
                2281701376UL,
                2281701376UL,
                2348810240UL,
                2348810240UL,
                2415919104UL,
                2415919104UL,
                2483027968UL,
                2483027968UL,
                2550136832UL,
                2550136832UL,
                2617245696UL,
                2617245696UL,
                2684354560UL,
                2684354560UL,
                2751463424UL,
                2751463424UL,
                2818572288UL,
                2818572288UL,
                2885681152UL,
                2885681152UL,
                2952790016UL,
                2952790016UL,
                3019898880UL,
                3019898880UL,
                3087007744UL,
                3087007744UL,
                3154116608UL,
                3154116608UL,
                3221225472UL,
                3221225472UL,
                3288334336UL,
                3288334336UL,
                3355443200UL,
                3355443200UL,
                3422552064UL,
                3422552064UL,
                3489660928UL,
                3489660928UL,
                3556769792UL,
                3556769792UL,
                3623878656UL,
                3623878656UL,
                3690987520UL,
                3690987520UL,
                3758096384UL,
                3758096384UL,
                3825205248UL,
                3825205248UL,
                3892314112UL,
                3892314112UL,
                3959422976UL,
                3959422976UL,
                4026531840UL,
                4026531840UL,
                4093640704UL,
                4093640704UL,
                4160749568UL,
                4160749568UL,
                4227858432UL,
                4227858432UL,
                0UL,
                0UL,
                67108864UL,
                67108864UL,
                134217728UL,
                134217728UL,
                201326592UL,
                201326592UL,
                268435456UL,
                268435456UL,
                335544320UL,
                335544320UL,
                402653184UL,
                402653184UL,
                469762048UL,
                469762048UL,
                536870912UL,
                536870912UL,
                603979776UL,
                603979776UL,
                671088640UL,
                671088640UL,
                738197504UL,
                738197504UL,
                805306368UL,
                805306368UL,
                872415232UL,
                872415232UL,
                939524096UL,
                939524096UL,
                1006632960UL,
                1006632960UL,
                1073741824UL,
                1073741824UL,
                1140850688UL,
                1140850688UL,
                1207959552UL,
                1207959552UL,
                1275068416UL,
                1275068416UL,
                1342177280UL,
                1342177280UL,
                1409286144UL,
                1409286144UL,
                1476395008UL,
                1476395008UL,
                1543503872UL,
                1543503872UL,
                1610612736UL,
                1610612736UL,
                1677721600UL,
                1677721600UL,
                1744830464UL,
                1744830464UL,
                1811939328UL,
                1811939328UL,
                1879048192UL,
                1879048192UL,
                1946157056UL,
                1946157056UL,
                2013265920UL,
                2013265920UL,
                2080374784UL,
                2080374784UL,
                2147483648UL,
                2147483648UL,
                2214592512UL,
                2214592512UL,
                2281701376UL,
                2281701376UL,
                2348810240UL,
                2348810240UL,
                2415919104UL,
                2415919104UL,
                2483027968UL,
                2483027968UL,
                2550136832UL,
                2550136832UL,
                2617245696UL,
                2617245696UL,
                2684354560UL,
                2684354560UL,
                2751463424UL,
                2751463424UL,
                2818572288UL,
                2818572288UL,
                2885681152UL,
                2885681152UL,
                2952790016UL,
                2952790016UL,
                3019898880UL,
                3019898880UL,
                3087007744UL,
                3087007744UL,
                3154116608UL,
                3154116608UL,
                3221225472UL,
                3221225472UL,
                3288334336UL,
                3288334336UL,
                3355443200UL,
                3355443200UL,
                3422552064UL,
                3422552064UL,
                3489660928UL,
                3489660928UL,
                3556769792UL,
                3556769792UL,
                3623878656UL,
                3623878656UL,
                3690987520UL,
                3690987520UL,
                3758096384UL,
                3758096384UL,
                3825205248UL,
                3825205248UL,
                3892314112UL,
                3892314112UL,
                3959422976UL,
                3959422976UL,
                4026531840UL,
                4026531840UL,
                4093640704UL,
                4093640704UL,
                4160749568UL,
                4160749568UL,
                4227858432UL,
                4227858432UL
            },
            {
                0UL,
                33554432UL,
                67108864UL,
                100663296UL,
                134217728UL,
                167772160UL,
                201326592UL,
                234881024UL,
                268435456UL,
                301989888UL,
                335544320UL,
                369098752UL,
                402653184UL,
                436207616UL,
                469762048UL,
                503316480UL,
                536870912UL,
                570425344UL,
                603979776UL,
                637534208UL,
                671088640UL,
                704643072UL,
                738197504UL,
                771751936UL,
                805306368UL,
                838860800UL,
                872415232UL,
                905969664UL,
                939524096UL,
                973078528UL,
                1006632960UL,
                1040187392UL,
                1073741824UL,
                1107296256UL,
                1140850688UL,
                1174405120UL,
                1207959552UL,
                1241513984UL,
                1275068416UL,
                1308622848UL,
                1342177280UL,
                1375731712UL,
                1409286144UL,
                1442840576UL,
                1476395008UL,
                1509949440UL,
                1543503872UL,
                1577058304UL,
                1610612736UL,
                1644167168UL,
                1677721600UL,
                1711276032UL,
                1744830464UL,
                1778384896UL,
                1811939328UL,
                1845493760UL,
                1879048192UL,
                1912602624UL,
                1946157056UL,
                1979711488UL,
                2013265920UL,
                2046820352UL,
                2080374784UL,
                2113929216UL,
                2147483648UL,
                2181038080UL,
                2214592512UL,
                2248146944UL,
                2281701376UL,
                2315255808UL,
                2348810240UL,
                2382364672UL,
                2415919104UL,
                2449473536UL,
                2483027968UL,
                2516582400UL,
                2550136832UL,
                2583691264UL,
                2617245696UL,
                2650800128UL,
                2684354560UL,
                2717908992UL,
                2751463424UL,
                2785017856UL,
                2818572288UL,
                2852126720UL,
                2885681152UL,
                2919235584UL,
                2952790016UL,
                2986344448UL,
                3019898880UL,
                3053453312UL,
                3087007744UL,
                3120562176UL,
                3154116608UL,
                3187671040UL,
                3221225472UL,
                3254779904UL,
                3288334336UL,
                3321888768UL,
                3355443200UL,
                3388997632UL,
                3422552064UL,
                3456106496UL,
                3489660928UL,
                3523215360UL,
                3556769792UL,
                3590324224UL,
                3623878656UL,
                3657433088UL,
                3690987520UL,
                3724541952UL,
                3758096384UL,
                3791650816UL,
                3825205248UL,
                3858759680UL,
                3892314112UL,
                3925868544UL,
                3959422976UL,
                3992977408UL,
                4026531840UL,
                4060086272UL,
                4093640704UL,
                4127195136UL,
                4160749568UL,
                4194304000UL,
                4227858432UL,
                4261412864UL,
                0UL,
                33554432UL,
                67108864UL,
                100663296UL,
                134217728UL,
                167772160UL,
                201326592UL,
                234881024UL,
                268435456UL,
                301989888UL,
                335544320UL,
                369098752UL,
                402653184UL,
                436207616UL,
                469762048UL,
                503316480UL,
                536870912UL,
                570425344UL,
                603979776UL,
                637534208UL,
                671088640UL,
                704643072UL,
                738197504UL,
                771751936UL,
                805306368UL,
                838860800UL,
                872415232UL,
                905969664UL,
                939524096UL,
                973078528UL,
                1006632960UL,
                1040187392UL,
                1073741824UL,
                1107296256UL,
                1140850688UL,
                1174405120UL,
                1207959552UL,
                1241513984UL,
                1275068416UL,
                1308622848UL,
                1342177280UL,
                1375731712UL,
                1409286144UL,
                1442840576UL,
                1476395008UL,
                1509949440UL,
                1543503872UL,
                1577058304UL,
                1610612736UL,
                1644167168UL,
                1677721600UL,
                1711276032UL,
                1744830464UL,
                1778384896UL,
                1811939328UL,
                1845493760UL,
                1879048192UL,
                1912602624UL,
                1946157056UL,
                1979711488UL,
                2013265920UL,
                2046820352UL,
                2080374784UL,
                2113929216UL,
                2147483648UL,
                2181038080UL,
                2214592512UL,
                2248146944UL,
                2281701376UL,
                2315255808UL,
                2348810240UL,
                2382364672UL,
                2415919104UL,
                2449473536UL,
                2483027968UL,
                2516582400UL,
                2550136832UL,
                2583691264UL,
                2617245696UL,
                2650800128UL,
                2684354560UL,
                2717908992UL,
                2751463424UL,
                2785017856UL,
                2818572288UL,
                2852126720UL,
                2885681152UL,
                2919235584UL,
                2952790016UL,
                2986344448UL,
                3019898880UL,
                3053453312UL,
                3087007744UL,
                3120562176UL,
                3154116608UL,
                3187671040UL,
                3221225472UL,
                3254779904UL,
                3288334336UL,
                3321888768UL,
                3355443200UL,
                3388997632UL,
                3422552064UL,
                3456106496UL,
                3489660928UL,
                3523215360UL,
                3556769792UL,
                3590324224UL,
                3623878656UL,
                3657433088UL,
                3690987520UL,
                3724541952UL,
                3758096384UL,
                3791650816UL,
                3825205248UL,
                3858759680UL,
                3892314112UL,
                3925868544UL,
                3959422976UL,
                3992977408UL,
                4026531840UL,
                4060086272UL,
                4093640704UL,
                4127195136UL,
                4160749568UL,
                4194304000UL,
                4227858432UL,
                4261412864UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL
            },
            {
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                0UL,
                1073741824UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL,
                2147483648UL,
                3221225472UL
            },
            {
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                0UL,
                0UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2013265920UL,
                2080374784UL,
                0UL,
                67108864UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4160749568UL,
                4227858432UL,
                2147483648UL,
                2214592512UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                67108864UL,
                67108864UL,
                134217728UL,
                134217728UL,
                201326592UL,
                201326592UL,
                268435456UL,
                268435456UL,
                335544320UL,
                335544320UL,
                402653184UL,
                402653184UL,
                469762048UL,
                469762048UL,
                536870912UL,
                536870912UL,
                603979776UL,
                603979776UL,
                671088640UL,
                671088640UL,
                738197504UL,
                738197504UL,
                805306368UL,
                805306368UL,
                872415232UL,
                872415232UL,
                939524096UL,
                939524096UL,
                1006632960UL,
                1006632960UL,
                1073741824UL,
                1073741824UL,
                1140850688UL,
                1140850688UL,
                1207959552UL,
                1207959552UL,
                1275068416UL,
                1275068416UL,
                1342177280UL,
                1342177280UL,
                1409286144UL,
                1409286144UL,
                1476395008UL,
                1476395008UL,
                1543503872UL,
                1543503872UL,
                1610612736UL,
                1610612736UL,
                1677721600UL,
                1677721600UL,
                1744830464UL,
                1744830464UL,
                1811939328UL,
                1811939328UL,
                1879048192UL,
                1879048192UL,
                1946157056UL,
                1946157056UL,
                2013265920UL,
                2013265920UL,
                2080374784UL,
                2080374784UL,
                0UL,
                0UL,
                67108864UL,
                67108864UL,
                134217728UL,
                134217728UL,
                201326592UL,
                201326592UL,
                268435456UL,
                268435456UL,
                335544320UL,
                335544320UL,
                402653184UL,
                402653184UL,
                469762048UL,
                469762048UL,
                536870912UL,
                536870912UL,
                603979776UL,
                603979776UL,
                671088640UL,
                671088640UL,
                738197504UL,
                738197504UL,
                805306368UL,
                805306368UL,
                872415232UL,
                872415232UL,
                939524096UL,
                939524096UL,
                1006632960UL,
                1006632960UL,
                1073741824UL,
                1073741824UL,
                1140850688UL,
                1140850688UL,
                1207959552UL,
                1207959552UL,
                1275068416UL,
                1275068416UL,
                1342177280UL,
                1342177280UL,
                1409286144UL,
                1409286144UL,
                1476395008UL,
                1476395008UL,
                1543503872UL,
                1543503872UL,
                1610612736UL,
                1610612736UL,
                1677721600UL,
                1677721600UL,
                1744830464UL,
                1744830464UL,
                1811939328UL,
                1811939328UL,
                1879048192UL,
                1879048192UL,
                1946157056UL,
                1946157056UL,
                2013265920UL,
                2013265920UL,
                2080374784UL,
                2080374784UL,
                2147483648UL,
                2147483648UL,
                2214592512UL,
                2214592512UL,
                2281701376UL,
                2281701376UL,
                2348810240UL,
                2348810240UL,
                2415919104UL,
                2415919104UL,
                2483027968UL,
                2483027968UL,
                2550136832UL,
                2550136832UL,
                2617245696UL,
                2617245696UL,
                2684354560UL,
                2684354560UL,
                2751463424UL,
                2751463424UL,
                2818572288UL,
                2818572288UL,
                2885681152UL,
                2885681152UL,
                2952790016UL,
                2952790016UL,
                3019898880UL,
                3019898880UL,
                3087007744UL,
                3087007744UL,
                3154116608UL,
                3154116608UL,
                3221225472UL,
                3221225472UL,
                3288334336UL,
                3288334336UL,
                3355443200UL,
                3355443200UL,
                3422552064UL,
                3422552064UL,
                3489660928UL,
                3489660928UL,
                3556769792UL,
                3556769792UL,
                3623878656UL,
                3623878656UL,
                3690987520UL,
                3690987520UL,
                3758096384UL,
                3758096384UL,
                3825205248UL,
                3825205248UL,
                3892314112UL,
                3892314112UL,
                3959422976UL,
                3959422976UL,
                4026531840UL,
                4026531840UL,
                4093640704UL,
                4093640704UL,
                4160749568UL,
                4160749568UL,
                4227858432UL,
                4227858432UL,
                2147483648UL,
                2147483648UL,
                2214592512UL,
                2214592512UL,
                2281701376UL,
                2281701376UL,
                2348810240UL,
                2348810240UL,
                2415919104UL,
                2415919104UL,
                2483027968UL,
                2483027968UL,
                2550136832UL,
                2550136832UL,
                2617245696UL,
                2617245696UL,
                2684354560UL,
                2684354560UL,
                2751463424UL,
                2751463424UL,
                2818572288UL,
                2818572288UL,
                2885681152UL,
                2885681152UL,
                2952790016UL,
                2952790016UL,
                3019898880UL,
                3019898880UL,
                3087007744UL,
                3087007744UL,
                3154116608UL,
                3154116608UL,
                3221225472UL,
                3221225472UL,
                3288334336UL,
                3288334336UL,
                3355443200UL,
                3355443200UL,
                3422552064UL,
                3422552064UL,
                3489660928UL,
                3489660928UL,
                3556769792UL,
                3556769792UL,
                3623878656UL,
                3623878656UL,
                3690987520UL,
                3690987520UL,
                3758096384UL,
                3758096384UL,
                3825205248UL,
                3825205248UL,
                3892314112UL,
                3892314112UL,
                3959422976UL,
                3959422976UL,
                4026531840UL,
                4026531840UL,
                4093640704UL,
                4093640704UL,
                4160749568UL,
                4160749568UL,
                4227858432UL,
                4227858432UL
            },
            {
                0UL,
                33554432UL,
                67108864UL,
                100663296UL,
                134217728UL,
                167772160UL,
                201326592UL,
                234881024UL,
                268435456UL,
                301989888UL,
                335544320UL,
                369098752UL,
                402653184UL,
                436207616UL,
                469762048UL,
                503316480UL,
                536870912UL,
                570425344UL,
                603979776UL,
                637534208UL,
                671088640UL,
                704643072UL,
                738197504UL,
                771751936UL,
                805306368UL,
                838860800UL,
                872415232UL,
                905969664UL,
                939524096UL,
                973078528UL,
                1006632960UL,
                1040187392UL,
                1073741824UL,
                1107296256UL,
                1140850688UL,
                1174405120UL,
                1207959552UL,
                1241513984UL,
                1275068416UL,
                1308622848UL,
                1342177280UL,
                1375731712UL,
                1409286144UL,
                1442840576UL,
                1476395008UL,
                1509949440UL,
                1543503872UL,
                1577058304UL,
                1610612736UL,
                1644167168UL,
                1677721600UL,
                1711276032UL,
                1744830464UL,
                1778384896UL,
                1811939328UL,
                1845493760UL,
                1879048192UL,
                1912602624UL,
                1946157056UL,
                1979711488UL,
                2013265920UL,
                2046820352UL,
                2080374784UL,
                2113929216UL,
                0UL,
                33554432UL,
                67108864UL,
                100663296UL,
                134217728UL,
                167772160UL,
                201326592UL,
                234881024UL,
                268435456UL,
                301989888UL,
                335544320UL,
                369098752UL,
                402653184UL,
                436207616UL,
                469762048UL,
                503316480UL,
                536870912UL,
                570425344UL,
                603979776UL,
                637534208UL,
                671088640UL,
                704643072UL,
                738197504UL,
                771751936UL,
                805306368UL,
                838860800UL,
                872415232UL,
                905969664UL,
                939524096UL,
                973078528UL,
                1006632960UL,
                1040187392UL,
                1073741824UL,
                1107296256UL,
                1140850688UL,
                1174405120UL,
                1207959552UL,
                1241513984UL,
                1275068416UL,
                1308622848UL,
                1342177280UL,
                1375731712UL,
                1409286144UL,
                1442840576UL,
                1476395008UL,
                1509949440UL,
                1543503872UL,
                1577058304UL,
                1610612736UL,
                1644167168UL,
                1677721600UL,
                1711276032UL,
                1744830464UL,
                1778384896UL,
                1811939328UL,
                1845493760UL,
                1879048192UL,
                1912602624UL,
                1946157056UL,
                1979711488UL,
                2013265920UL,
                2046820352UL,
                2080374784UL,
                2113929216UL,
                2147483648UL,
                2181038080UL,
                2214592512UL,
                2248146944UL,
                2281701376UL,
                2315255808UL,
                2348810240UL,
                2382364672UL,
                2415919104UL,
                2449473536UL,
                2483027968UL,
                2516582400UL,
                2550136832UL,
                2583691264UL,
                2617245696UL,
                2650800128UL,
                2684354560UL,
                2717908992UL,
                2751463424UL,
                2785017856UL,
                2818572288UL,
                2852126720UL,
                2885681152UL,
                2919235584UL,
                2952790016UL,
                2986344448UL,
                3019898880UL,
                3053453312UL,
                3087007744UL,
                3120562176UL,
                3154116608UL,
                3187671040UL,
                3221225472UL,
                3254779904UL,
                3288334336UL,
                3321888768UL,
                3355443200UL,
                3388997632UL,
                3422552064UL,
                3456106496UL,
                3489660928UL,
                3523215360UL,
                3556769792UL,
                3590324224UL,
                3623878656UL,
                3657433088UL,
                3690987520UL,
                3724541952UL,
                3758096384UL,
                3791650816UL,
                3825205248UL,
                3858759680UL,
                3892314112UL,
                3925868544UL,
                3959422976UL,
                3992977408UL,
                4026531840UL,
                4060086272UL,
                4093640704UL,
                4127195136UL,
                4160749568UL,
                4194304000UL,
                4227858432UL,
                4261412864UL,
                2147483648UL,
                2181038080UL,
                2214592512UL,
                2248146944UL,
                2281701376UL,
                2315255808UL,
                2348810240UL,
                2382364672UL,
                2415919104UL,
                2449473536UL,
                2483027968UL,
                2516582400UL,
                2550136832UL,
                2583691264UL,
                2617245696UL,
                2650800128UL,
                2684354560UL,
                2717908992UL,
                2751463424UL,
                2785017856UL,
                2818572288UL,
                2852126720UL,
                2885681152UL,
                2919235584UL,
                2952790016UL,
                2986344448UL,
                3019898880UL,
                3053453312UL,
                3087007744UL,
                3120562176UL,
                3154116608UL,
                3187671040UL,
                3221225472UL,
                3254779904UL,
                3288334336UL,
                3321888768UL,
                3355443200UL,
                3388997632UL,
                3422552064UL,
                3456106496UL,
                3489660928UL,
                3523215360UL,
                3556769792UL,
                3590324224UL,
                3623878656UL,
                3657433088UL,
                3690987520UL,
                3724541952UL,
                3758096384UL,
                3791650816UL,
                3825205248UL,
                3858759680UL,
                3892314112UL,
                3925868544UL,
                3959422976UL,
                3992977408UL,
                4026531840UL,
                4060086272UL,
                4093640704UL,
                4127195136UL,
                4160749568UL,
                4194304000UL,
                4227858432UL,
                4261412864UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL
            },
            {
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                0UL,
                536870912UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                1073741824UL,
                1610612736UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                2147483648UL,
                2684354560UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL,
                3221225472UL,
                3758096384UL
            },
            {
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                939524096UL,
                939524096UL,
                0UL,
                0UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                939524096UL,
                1006632960UL,
                0UL,
                67108864UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2013265920UL,
                2080374784UL,
                1073741824UL,
                1140850688UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3087007744UL,
                3154116608UL,
                2147483648UL,
                2214592512UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4160749568UL,
                4227858432UL,
                3221225472UL,
                3288334336UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                67108864UL,
                67108864UL,
                134217728UL,
                134217728UL,
                201326592UL,
                201326592UL,
                268435456UL,
                268435456UL,
                335544320UL,
                335544320UL,
                402653184UL,
                402653184UL,
                469762048UL,
                469762048UL,
                536870912UL,
                536870912UL,
                603979776UL,
                603979776UL,
                671088640UL,
                671088640UL,
                738197504UL,
                738197504UL,
                805306368UL,
                805306368UL,
                872415232UL,
                872415232UL,
                939524096UL,
                939524096UL,
                1006632960UL,
                1006632960UL,
                0UL,
                0UL,
                67108864UL,
                67108864UL,
                134217728UL,
                134217728UL,
                201326592UL,
                201326592UL,
                268435456UL,
                268435456UL,
                335544320UL,
                335544320UL,
                402653184UL,
                402653184UL,
                469762048UL,
                469762048UL,
                536870912UL,
                536870912UL,
                603979776UL,
                603979776UL,
                671088640UL,
                671088640UL,
                738197504UL,
                738197504UL,
                805306368UL,
                805306368UL,
                872415232UL,
                872415232UL,
                939524096UL,
                939524096UL,
                1006632960UL,
                1006632960UL,
                1073741824UL,
                1073741824UL,
                1140850688UL,
                1140850688UL,
                1207959552UL,
                1207959552UL,
                1275068416UL,
                1275068416UL,
                1342177280UL,
                1342177280UL,
                1409286144UL,
                1409286144UL,
                1476395008UL,
                1476395008UL,
                1543503872UL,
                1543503872UL,
                1610612736UL,
                1610612736UL,
                1677721600UL,
                1677721600UL,
                1744830464UL,
                1744830464UL,
                1811939328UL,
                1811939328UL,
                1879048192UL,
                1879048192UL,
                1946157056UL,
                1946157056UL,
                2013265920UL,
                2013265920UL,
                2080374784UL,
                2080374784UL,
                1073741824UL,
                1073741824UL,
                1140850688UL,
                1140850688UL,
                1207959552UL,
                1207959552UL,
                1275068416UL,
                1275068416UL,
                1342177280UL,
                1342177280UL,
                1409286144UL,
                1409286144UL,
                1476395008UL,
                1476395008UL,
                1543503872UL,
                1543503872UL,
                1610612736UL,
                1610612736UL,
                1677721600UL,
                1677721600UL,
                1744830464UL,
                1744830464UL,
                1811939328UL,
                1811939328UL,
                1879048192UL,
                1879048192UL,
                1946157056UL,
                1946157056UL,
                2013265920UL,
                2013265920UL,
                2080374784UL,
                2080374784UL,
                2147483648UL,
                2147483648UL,
                2214592512UL,
                2214592512UL,
                2281701376UL,
                2281701376UL,
                2348810240UL,
                2348810240UL,
                2415919104UL,
                2415919104UL,
                2483027968UL,
                2483027968UL,
                2550136832UL,
                2550136832UL,
                2617245696UL,
                2617245696UL,
                2684354560UL,
                2684354560UL,
                2751463424UL,
                2751463424UL,
                2818572288UL,
                2818572288UL,
                2885681152UL,
                2885681152UL,
                2952790016UL,
                2952790016UL,
                3019898880UL,
                3019898880UL,
                3087007744UL,
                3087007744UL,
                3154116608UL,
                3154116608UL,
                2147483648UL,
                2147483648UL,
                2214592512UL,
                2214592512UL,
                2281701376UL,
                2281701376UL,
                2348810240UL,
                2348810240UL,
                2415919104UL,
                2415919104UL,
                2483027968UL,
                2483027968UL,
                2550136832UL,
                2550136832UL,
                2617245696UL,
                2617245696UL,
                2684354560UL,
                2684354560UL,
                2751463424UL,
                2751463424UL,
                2818572288UL,
                2818572288UL,
                2885681152UL,
                2885681152UL,
                2952790016UL,
                2952790016UL,
                3019898880UL,
                3019898880UL,
                3087007744UL,
                3087007744UL,
                3154116608UL,
                3154116608UL,
                3221225472UL,
                3221225472UL,
                3288334336UL,
                3288334336UL,
                3355443200UL,
                3355443200UL,
                3422552064UL,
                3422552064UL,
                3489660928UL,
                3489660928UL,
                3556769792UL,
                3556769792UL,
                3623878656UL,
                3623878656UL,
                3690987520UL,
                3690987520UL,
                3758096384UL,
                3758096384UL,
                3825205248UL,
                3825205248UL,
                3892314112UL,
                3892314112UL,
                3959422976UL,
                3959422976UL,
                4026531840UL,
                4026531840UL,
                4093640704UL,
                4093640704UL,
                4160749568UL,
                4160749568UL,
                4227858432UL,
                4227858432UL,
                3221225472UL,
                3221225472UL,
                3288334336UL,
                3288334336UL,
                3355443200UL,
                3355443200UL,
                3422552064UL,
                3422552064UL,
                3489660928UL,
                3489660928UL,
                3556769792UL,
                3556769792UL,
                3623878656UL,
                3623878656UL,
                3690987520UL,
                3690987520UL,
                3758096384UL,
                3758096384UL,
                3825205248UL,
                3825205248UL,
                3892314112UL,
                3892314112UL,
                3959422976UL,
                3959422976UL,
                4026531840UL,
                4026531840UL,
                4093640704UL,
                4093640704UL,
                4160749568UL,
                4160749568UL,
                4227858432UL,
                4227858432UL
            },
            {
                0UL,
                33554432UL,
                67108864UL,
                100663296UL,
                134217728UL,
                167772160UL,
                201326592UL,
                234881024UL,
                268435456UL,
                301989888UL,
                335544320UL,
                369098752UL,
                402653184UL,
                436207616UL,
                469762048UL,
                503316480UL,
                536870912UL,
                570425344UL,
                603979776UL,
                637534208UL,
                671088640UL,
                704643072UL,
                738197504UL,
                771751936UL,
                805306368UL,
                838860800UL,
                872415232UL,
                905969664UL,
                939524096UL,
                973078528UL,
                1006632960UL,
                1040187392UL,
                0UL,
                33554432UL,
                67108864UL,
                100663296UL,
                134217728UL,
                167772160UL,
                201326592UL,
                234881024UL,
                268435456UL,
                301989888UL,
                335544320UL,
                369098752UL,
                402653184UL,
                436207616UL,
                469762048UL,
                503316480UL,
                536870912UL,
                570425344UL,
                603979776UL,
                637534208UL,
                671088640UL,
                704643072UL,
                738197504UL,
                771751936UL,
                805306368UL,
                838860800UL,
                872415232UL,
                905969664UL,
                939524096UL,
                973078528UL,
                1006632960UL,
                1040187392UL,
                1073741824UL,
                1107296256UL,
                1140850688UL,
                1174405120UL,
                1207959552UL,
                1241513984UL,
                1275068416UL,
                1308622848UL,
                1342177280UL,
                1375731712UL,
                1409286144UL,
                1442840576UL,
                1476395008UL,
                1509949440UL,
                1543503872UL,
                1577058304UL,
                1610612736UL,
                1644167168UL,
                1677721600UL,
                1711276032UL,
                1744830464UL,
                1778384896UL,
                1811939328UL,
                1845493760UL,
                1879048192UL,
                1912602624UL,
                1946157056UL,
                1979711488UL,
                2013265920UL,
                2046820352UL,
                2080374784UL,
                2113929216UL,
                1073741824UL,
                1107296256UL,
                1140850688UL,
                1174405120UL,
                1207959552UL,
                1241513984UL,
                1275068416UL,
                1308622848UL,
                1342177280UL,
                1375731712UL,
                1409286144UL,
                1442840576UL,
                1476395008UL,
                1509949440UL,
                1543503872UL,
                1577058304UL,
                1610612736UL,
                1644167168UL,
                1677721600UL,
                1711276032UL,
                1744830464UL,
                1778384896UL,
                1811939328UL,
                1845493760UL,
                1879048192UL,
                1912602624UL,
                1946157056UL,
                1979711488UL,
                2013265920UL,
                2046820352UL,
                2080374784UL,
                2113929216UL,
                2147483648UL,
                2181038080UL,
                2214592512UL,
                2248146944UL,
                2281701376UL,
                2315255808UL,
                2348810240UL,
                2382364672UL,
                2415919104UL,
                2449473536UL,
                2483027968UL,
                2516582400UL,
                2550136832UL,
                2583691264UL,
                2617245696UL,
                2650800128UL,
                2684354560UL,
                2717908992UL,
                2751463424UL,
                2785017856UL,
                2818572288UL,
                2852126720UL,
                2885681152UL,
                2919235584UL,
                2952790016UL,
                2986344448UL,
                3019898880UL,
                3053453312UL,
                3087007744UL,
                3120562176UL,
                3154116608UL,
                3187671040UL,
                2147483648UL,
                2181038080UL,
                2214592512UL,
                2248146944UL,
                2281701376UL,
                2315255808UL,
                2348810240UL,
                2382364672UL,
                2415919104UL,
                2449473536UL,
                2483027968UL,
                2516582400UL,
                2550136832UL,
                2583691264UL,
                2617245696UL,
                2650800128UL,
                2684354560UL,
                2717908992UL,
                2751463424UL,
                2785017856UL,
                2818572288UL,
                2852126720UL,
                2885681152UL,
                2919235584UL,
                2952790016UL,
                2986344448UL,
                3019898880UL,
                3053453312UL,
                3087007744UL,
                3120562176UL,
                3154116608UL,
                3187671040UL,
                3221225472UL,
                3254779904UL,
                3288334336UL,
                3321888768UL,
                3355443200UL,
                3388997632UL,
                3422552064UL,
                3456106496UL,
                3489660928UL,
                3523215360UL,
                3556769792UL,
                3590324224UL,
                3623878656UL,
                3657433088UL,
                3690987520UL,
                3724541952UL,
                3758096384UL,
                3791650816UL,
                3825205248UL,
                3858759680UL,
                3892314112UL,
                3925868544UL,
                3959422976UL,
                3992977408UL,
                4026531840UL,
                4060086272UL,
                4093640704UL,
                4127195136UL,
                4160749568UL,
                4194304000UL,
                4227858432UL,
                4261412864UL,
                3221225472UL,
                3254779904UL,
                3288334336UL,
                3321888768UL,
                3355443200UL,
                3388997632UL,
                3422552064UL,
                3456106496UL,
                3489660928UL,
                3523215360UL,
                3556769792UL,
                3590324224UL,
                3623878656UL,
                3657433088UL,
                3690987520UL,
                3724541952UL,
                3758096384UL,
                3791650816UL,
                3825205248UL,
                3858759680UL,
                3892314112UL,
                3925868544UL,
                3959422976UL,
                3992977408UL,
                4026531840UL,
                4060086272UL,
                4093640704UL,
                4127195136UL,
                4160749568UL,
                4194304000UL,
                4227858432UL,
                4261412864UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL
            },
            {
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                0UL,
                268435456UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                536870912UL,
                805306368UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1073741824UL,
                1342177280UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                1610612736UL,
                1879048192UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2147483648UL,
                2415919104UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                2684354560UL,
                2952790016UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3221225472UL,
                3489660928UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL,
                3758096384UL,
                4026531840UL
            },
            {
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                402653184UL,
                402653184UL,
                0UL,
                0UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                939524096UL,
                939524096UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                402653184UL,
                469762048UL,
                0UL,
                67108864UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                939524096UL,
                1006632960UL,
                536870912UL,
                603979776UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1476395008UL,
                1543503872UL,
                1073741824UL,
                1140850688UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2013265920UL,
                2080374784UL,
                1610612736UL,
                1677721600UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2550136832UL,
                2617245696UL,
                2147483648UL,
                2214592512UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3087007744UL,
                3154116608UL,
                2684354560UL,
                2751463424UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3623878656UL,
                3690987520UL,
                3221225472UL,
                3288334336UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4160749568UL,
                4227858432UL,
                3758096384UL,
                3825205248UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                67108864UL,
                67108864UL,
                134217728UL,
                134217728UL,
                201326592UL,
                201326592UL,
                268435456UL,
                268435456UL,
                335544320UL,
                335544320UL,
                402653184UL,
                402653184UL,
                469762048UL,
                469762048UL,
                0UL,
                0UL,
                67108864UL,
                67108864UL,
                134217728UL,
                134217728UL,
                201326592UL,
                201326592UL,
                268435456UL,
                268435456UL,
                335544320UL,
                335544320UL,
                402653184UL,
                402653184UL,
                469762048UL,
                469762048UL,
                536870912UL,
                536870912UL,
                603979776UL,
                603979776UL,
                671088640UL,
                671088640UL,
                738197504UL,
                738197504UL,
                805306368UL,
                805306368UL,
                872415232UL,
                872415232UL,
                939524096UL,
                939524096UL,
                1006632960UL,
                1006632960UL,
                536870912UL,
                536870912UL,
                603979776UL,
                603979776UL,
                671088640UL,
                671088640UL,
                738197504UL,
                738197504UL,
                805306368UL,
                805306368UL,
                872415232UL,
                872415232UL,
                939524096UL,
                939524096UL,
                1006632960UL,
                1006632960UL,
                1073741824UL,
                1073741824UL,
                1140850688UL,
                1140850688UL,
                1207959552UL,
                1207959552UL,
                1275068416UL,
                1275068416UL,
                1342177280UL,
                1342177280UL,
                1409286144UL,
                1409286144UL,
                1476395008UL,
                1476395008UL,
                1543503872UL,
                1543503872UL,
                1073741824UL,
                1073741824UL,
                1140850688UL,
                1140850688UL,
                1207959552UL,
                1207959552UL,
                1275068416UL,
                1275068416UL,
                1342177280UL,
                1342177280UL,
                1409286144UL,
                1409286144UL,
                1476395008UL,
                1476395008UL,
                1543503872UL,
                1543503872UL,
                1610612736UL,
                1610612736UL,
                1677721600UL,
                1677721600UL,
                1744830464UL,
                1744830464UL,
                1811939328UL,
                1811939328UL,
                1879048192UL,
                1879048192UL,
                1946157056UL,
                1946157056UL,
                2013265920UL,
                2013265920UL,
                2080374784UL,
                2080374784UL,
                1610612736UL,
                1610612736UL,
                1677721600UL,
                1677721600UL,
                1744830464UL,
                1744830464UL,
                1811939328UL,
                1811939328UL,
                1879048192UL,
                1879048192UL,
                1946157056UL,
                1946157056UL,
                2013265920UL,
                2013265920UL,
                2080374784UL,
                2080374784UL,
                2147483648UL,
                2147483648UL,
                2214592512UL,
                2214592512UL,
                2281701376UL,
                2281701376UL,
                2348810240UL,
                2348810240UL,
                2415919104UL,
                2415919104UL,
                2483027968UL,
                2483027968UL,
                2550136832UL,
                2550136832UL,
                2617245696UL,
                2617245696UL,
                2147483648UL,
                2147483648UL,
                2214592512UL,
                2214592512UL,
                2281701376UL,
                2281701376UL,
                2348810240UL,
                2348810240UL,
                2415919104UL,
                2415919104UL,
                2483027968UL,
                2483027968UL,
                2550136832UL,
                2550136832UL,
                2617245696UL,
                2617245696UL,
                2684354560UL,
                2684354560UL,
                2751463424UL,
                2751463424UL,
                2818572288UL,
                2818572288UL,
                2885681152UL,
                2885681152UL,
                2952790016UL,
                2952790016UL,
                3019898880UL,
                3019898880UL,
                3087007744UL,
                3087007744UL,
                3154116608UL,
                3154116608UL,
                2684354560UL,
                2684354560UL,
                2751463424UL,
                2751463424UL,
                2818572288UL,
                2818572288UL,
                2885681152UL,
                2885681152UL,
                2952790016UL,
                2952790016UL,
                3019898880UL,
                3019898880UL,
                3087007744UL,
                3087007744UL,
                3154116608UL,
                3154116608UL,
                3221225472UL,
                3221225472UL,
                3288334336UL,
                3288334336UL,
                3355443200UL,
                3355443200UL,
                3422552064UL,
                3422552064UL,
                3489660928UL,
                3489660928UL,
                3556769792UL,
                3556769792UL,
                3623878656UL,
                3623878656UL,
                3690987520UL,
                3690987520UL,
                3221225472UL,
                3221225472UL,
                3288334336UL,
                3288334336UL,
                3355443200UL,
                3355443200UL,
                3422552064UL,
                3422552064UL,
                3489660928UL,
                3489660928UL,
                3556769792UL,
                3556769792UL,
                3623878656UL,
                3623878656UL,
                3690987520UL,
                3690987520UL,
                3758096384UL,
                3758096384UL,
                3825205248UL,
                3825205248UL,
                3892314112UL,
                3892314112UL,
                3959422976UL,
                3959422976UL,
                4026531840UL,
                4026531840UL,
                4093640704UL,
                4093640704UL,
                4160749568UL,
                4160749568UL,
                4227858432UL,
                4227858432UL,
                3758096384UL,
                3758096384UL,
                3825205248UL,
                3825205248UL,
                3892314112UL,
                3892314112UL,
                3959422976UL,
                3959422976UL,
                4026531840UL,
                4026531840UL,
                4093640704UL,
                4093640704UL,
                4160749568UL,
                4160749568UL,
                4227858432UL,
                4227858432UL
            },
            {
                0UL,
                33554432UL,
                67108864UL,
                100663296UL,
                134217728UL,
                167772160UL,
                201326592UL,
                234881024UL,
                268435456UL,
                301989888UL,
                335544320UL,
                369098752UL,
                402653184UL,
                436207616UL,
                469762048UL,
                503316480UL,
                0UL,
                33554432UL,
                67108864UL,
                100663296UL,
                134217728UL,
                167772160UL,
                201326592UL,
                234881024UL,
                268435456UL,
                301989888UL,
                335544320UL,
                369098752UL,
                402653184UL,
                436207616UL,
                469762048UL,
                503316480UL,
                536870912UL,
                570425344UL,
                603979776UL,
                637534208UL,
                671088640UL,
                704643072UL,
                738197504UL,
                771751936UL,
                805306368UL,
                838860800UL,
                872415232UL,
                905969664UL,
                939524096UL,
                973078528UL,
                1006632960UL,
                1040187392UL,
                536870912UL,
                570425344UL,
                603979776UL,
                637534208UL,
                671088640UL,
                704643072UL,
                738197504UL,
                771751936UL,
                805306368UL,
                838860800UL,
                872415232UL,
                905969664UL,
                939524096UL,
                973078528UL,
                1006632960UL,
                1040187392UL,
                1073741824UL,
                1107296256UL,
                1140850688UL,
                1174405120UL,
                1207959552UL,
                1241513984UL,
                1275068416UL,
                1308622848UL,
                1342177280UL,
                1375731712UL,
                1409286144UL,
                1442840576UL,
                1476395008UL,
                1509949440UL,
                1543503872UL,
                1577058304UL,
                1073741824UL,
                1107296256UL,
                1140850688UL,
                1174405120UL,
                1207959552UL,
                1241513984UL,
                1275068416UL,
                1308622848UL,
                1342177280UL,
                1375731712UL,
                1409286144UL,
                1442840576UL,
                1476395008UL,
                1509949440UL,
                1543503872UL,
                1577058304UL,
                1610612736UL,
                1644167168UL,
                1677721600UL,
                1711276032UL,
                1744830464UL,
                1778384896UL,
                1811939328UL,
                1845493760UL,
                1879048192UL,
                1912602624UL,
                1946157056UL,
                1979711488UL,
                2013265920UL,
                2046820352UL,
                2080374784UL,
                2113929216UL,
                1610612736UL,
                1644167168UL,
                1677721600UL,
                1711276032UL,
                1744830464UL,
                1778384896UL,
                1811939328UL,
                1845493760UL,
                1879048192UL,
                1912602624UL,
                1946157056UL,
                1979711488UL,
                2013265920UL,
                2046820352UL,
                2080374784UL,
                2113929216UL,
                2147483648UL,
                2181038080UL,
                2214592512UL,
                2248146944UL,
                2281701376UL,
                2315255808UL,
                2348810240UL,
                2382364672UL,
                2415919104UL,
                2449473536UL,
                2483027968UL,
                2516582400UL,
                2550136832UL,
                2583691264UL,
                2617245696UL,
                2650800128UL,
                2147483648UL,
                2181038080UL,
                2214592512UL,
                2248146944UL,
                2281701376UL,
                2315255808UL,
                2348810240UL,
                2382364672UL,
                2415919104UL,
                2449473536UL,
                2483027968UL,
                2516582400UL,
                2550136832UL,
                2583691264UL,
                2617245696UL,
                2650800128UL,
                2684354560UL,
                2717908992UL,
                2751463424UL,
                2785017856UL,
                2818572288UL,
                2852126720UL,
                2885681152UL,
                2919235584UL,
                2952790016UL,
                2986344448UL,
                3019898880UL,
                3053453312UL,
                3087007744UL,
                3120562176UL,
                3154116608UL,
                3187671040UL,
                2684354560UL,
                2717908992UL,
                2751463424UL,
                2785017856UL,
                2818572288UL,
                2852126720UL,
                2885681152UL,
                2919235584UL,
                2952790016UL,
                2986344448UL,
                3019898880UL,
                3053453312UL,
                3087007744UL,
                3120562176UL,
                3154116608UL,
                3187671040UL,
                3221225472UL,
                3254779904UL,
                3288334336UL,
                3321888768UL,
                3355443200UL,
                3388997632UL,
                3422552064UL,
                3456106496UL,
                3489660928UL,
                3523215360UL,
                3556769792UL,
                3590324224UL,
                3623878656UL,
                3657433088UL,
                3690987520UL,
                3724541952UL,
                3221225472UL,
                3254779904UL,
                3288334336UL,
                3321888768UL,
                3355443200UL,
                3388997632UL,
                3422552064UL,
                3456106496UL,
                3489660928UL,
                3523215360UL,
                3556769792UL,
                3590324224UL,
                3623878656UL,
                3657433088UL,
                3690987520UL,
                3724541952UL,
                3758096384UL,
                3791650816UL,
                3825205248UL,
                3858759680UL,
                3892314112UL,
                3925868544UL,
                3959422976UL,
                3992977408UL,
                4026531840UL,
                4060086272UL,
                4093640704UL,
                4127195136UL,
                4160749568UL,
                4194304000UL,
                4227858432UL,
                4261412864UL,
                3758096384UL,
                3791650816UL,
                3825205248UL,
                3858759680UL,
                3892314112UL,
                3925868544UL,
                3959422976UL,
                3992977408UL,
                4026531840UL,
                4060086272UL,
                4093640704UL,
                4127195136UL,
                4160749568UL,
                4194304000UL,
                4227858432UL,
                4261412864UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL
            },
            {
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                0UL,
                134217728UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                268435456UL,
                402653184UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                536870912UL,
                671088640UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                805306368UL,
                939524096UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1073741824UL,
                1207959552UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1342177280UL,
                1476395008UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1610612736UL,
                1744830464UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                1879048192UL,
                2013265920UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2147483648UL,
                2281701376UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2415919104UL,
                2550136832UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2684354560UL,
                2818572288UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                2952790016UL,
                3087007744UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3221225472UL,
                3355443200UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3489660928UL,
                3623878656UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                3758096384UL,
                3892314112UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL,
                4026531840UL,
                4160749568UL
            },
            {
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                134217728UL,
                134217728UL,
                0UL,
                0UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                402653184UL,
                402653184UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                671088640UL,
                671088640UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                939524096UL,
                939524096UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                134217728UL,
                201326592UL,
                0UL,
                67108864UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                402653184UL,
                469762048UL,
                268435456UL,
                335544320UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                671088640UL,
                738197504UL,
                536870912UL,
                603979776UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                939524096UL,
                1006632960UL,
                805306368UL,
                872415232UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1207959552UL,
                1275068416UL,
                1073741824UL,
                1140850688UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1476395008UL,
                1543503872UL,
                1342177280UL,
                1409286144UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1744830464UL,
                1811939328UL,
                1610612736UL,
                1677721600UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2013265920UL,
                2080374784UL,
                1879048192UL,
                1946157056UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2281701376UL,
                2348810240UL,
                2147483648UL,
                2214592512UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2550136832UL,
                2617245696UL,
                2415919104UL,
                2483027968UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2818572288UL,
                2885681152UL,
                2684354560UL,
                2751463424UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3087007744UL,
                3154116608UL,
                2952790016UL,
                3019898880UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3355443200UL,
                3422552064UL,
                3221225472UL,
                3288334336UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3623878656UL,
                3690987520UL,
                3489660928UL,
                3556769792UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3892314112UL,
                3959422976UL,
                3758096384UL,
                3825205248UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4160749568UL,
                4227858432UL,
                4026531840UL,
                4093640704UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                67108864UL,
                67108864UL,
                134217728UL,
                134217728UL,
                201326592UL,
                201326592UL,
                0UL,
                0UL,
                67108864UL,
                67108864UL,
                134217728UL,
                134217728UL,
                201326592UL,
                201326592UL,
                268435456UL,
                268435456UL,
                335544320UL,
                335544320UL,
                402653184UL,
                402653184UL,
                469762048UL,
                469762048UL,
                268435456UL,
                268435456UL,
                335544320UL,
                335544320UL,
                402653184UL,
                402653184UL,
                469762048UL,
                469762048UL,
                536870912UL,
                536870912UL,
                603979776UL,
                603979776UL,
                671088640UL,
                671088640UL,
                738197504UL,
                738197504UL,
                536870912UL,
                536870912UL,
                603979776UL,
                603979776UL,
                671088640UL,
                671088640UL,
                738197504UL,
                738197504UL,
                805306368UL,
                805306368UL,
                872415232UL,
                872415232UL,
                939524096UL,
                939524096UL,
                1006632960UL,
                1006632960UL,
                805306368UL,
                805306368UL,
                872415232UL,
                872415232UL,
                939524096UL,
                939524096UL,
                1006632960UL,
                1006632960UL,
                1073741824UL,
                1073741824UL,
                1140850688UL,
                1140850688UL,
                1207959552UL,
                1207959552UL,
                1275068416UL,
                1275068416UL,
                1073741824UL,
                1073741824UL,
                1140850688UL,
                1140850688UL,
                1207959552UL,
                1207959552UL,
                1275068416UL,
                1275068416UL,
                1342177280UL,
                1342177280UL,
                1409286144UL,
                1409286144UL,
                1476395008UL,
                1476395008UL,
                1543503872UL,
                1543503872UL,
                1342177280UL,
                1342177280UL,
                1409286144UL,
                1409286144UL,
                1476395008UL,
                1476395008UL,
                1543503872UL,
                1543503872UL,
                1610612736UL,
                1610612736UL,
                1677721600UL,
                1677721600UL,
                1744830464UL,
                1744830464UL,
                1811939328UL,
                1811939328UL,
                1610612736UL,
                1610612736UL,
                1677721600UL,
                1677721600UL,
                1744830464UL,
                1744830464UL,
                1811939328UL,
                1811939328UL,
                1879048192UL,
                1879048192UL,
                1946157056UL,
                1946157056UL,
                2013265920UL,
                2013265920UL,
                2080374784UL,
                2080374784UL,
                1879048192UL,
                1879048192UL,
                1946157056UL,
                1946157056UL,
                2013265920UL,
                2013265920UL,
                2080374784UL,
                2080374784UL,
                2147483648UL,
                2147483648UL,
                2214592512UL,
                2214592512UL,
                2281701376UL,
                2281701376UL,
                2348810240UL,
                2348810240UL,
                2147483648UL,
                2147483648UL,
                2214592512UL,
                2214592512UL,
                2281701376UL,
                2281701376UL,
                2348810240UL,
                2348810240UL,
                2415919104UL,
                2415919104UL,
                2483027968UL,
                2483027968UL,
                2550136832UL,
                2550136832UL,
                2617245696UL,
                2617245696UL,
                2415919104UL,
                2415919104UL,
                2483027968UL,
                2483027968UL,
                2550136832UL,
                2550136832UL,
                2617245696UL,
                2617245696UL,
                2684354560UL,
                2684354560UL,
                2751463424UL,
                2751463424UL,
                2818572288UL,
                2818572288UL,
                2885681152UL,
                2885681152UL,
                2684354560UL,
                2684354560UL,
                2751463424UL,
                2751463424UL,
                2818572288UL,
                2818572288UL,
                2885681152UL,
                2885681152UL,
                2952790016UL,
                2952790016UL,
                3019898880UL,
                3019898880UL,
                3087007744UL,
                3087007744UL,
                3154116608UL,
                3154116608UL,
                2952790016UL,
                2952790016UL,
                3019898880UL,
                3019898880UL,
                3087007744UL,
                3087007744UL,
                3154116608UL,
                3154116608UL,
                3221225472UL,
                3221225472UL,
                3288334336UL,
                3288334336UL,
                3355443200UL,
                3355443200UL,
                3422552064UL,
                3422552064UL,
                3221225472UL,
                3221225472UL,
                3288334336UL,
                3288334336UL,
                3355443200UL,
                3355443200UL,
                3422552064UL,
                3422552064UL,
                3489660928UL,
                3489660928UL,
                3556769792UL,
                3556769792UL,
                3623878656UL,
                3623878656UL,
                3690987520UL,
                3690987520UL,
                3489660928UL,
                3489660928UL,
                3556769792UL,
                3556769792UL,
                3623878656UL,
                3623878656UL,
                3690987520UL,
                3690987520UL,
                3758096384UL,
                3758096384UL,
                3825205248UL,
                3825205248UL,
                3892314112UL,
                3892314112UL,
                3959422976UL,
                3959422976UL,
                3758096384UL,
                3758096384UL,
                3825205248UL,
                3825205248UL,
                3892314112UL,
                3892314112UL,
                3959422976UL,
                3959422976UL,
                4026531840UL,
                4026531840UL,
                4093640704UL,
                4093640704UL,
                4160749568UL,
                4160749568UL,
                4227858432UL,
                4227858432UL,
                4026531840UL,
                4026531840UL,
                4093640704UL,
                4093640704UL,
                4160749568UL,
                4160749568UL,
                4227858432UL,
                4227858432UL
            },
            {
                0UL,
                33554432UL,
                67108864UL,
                100663296UL,
                134217728UL,
                167772160UL,
                201326592UL,
                234881024UL,
                0UL,
                33554432UL,
                67108864UL,
                100663296UL,
                134217728UL,
                167772160UL,
                201326592UL,
                234881024UL,
                268435456UL,
                301989888UL,
                335544320UL,
                369098752UL,
                402653184UL,
                436207616UL,
                469762048UL,
                503316480UL,
                268435456UL,
                301989888UL,
                335544320UL,
                369098752UL,
                402653184UL,
                436207616UL,
                469762048UL,
                503316480UL,
                536870912UL,
                570425344UL,
                603979776UL,
                637534208UL,
                671088640UL,
                704643072UL,
                738197504UL,
                771751936UL,
                536870912UL,
                570425344UL,
                603979776UL,
                637534208UL,
                671088640UL,
                704643072UL,
                738197504UL,
                771751936UL,
                805306368UL,
                838860800UL,
                872415232UL,
                905969664UL,
                939524096UL,
                973078528UL,
                1006632960UL,
                1040187392UL,
                805306368UL,
                838860800UL,
                872415232UL,
                905969664UL,
                939524096UL,
                973078528UL,
                1006632960UL,
                1040187392UL,
                1073741824UL,
                1107296256UL,
                1140850688UL,
                1174405120UL,
                1207959552UL,
                1241513984UL,
                1275068416UL,
                1308622848UL,
                1073741824UL,
                1107296256UL,
                1140850688UL,
                1174405120UL,
                1207959552UL,
                1241513984UL,
                1275068416UL,
                1308622848UL,
                1342177280UL,
                1375731712UL,
                1409286144UL,
                1442840576UL,
                1476395008UL,
                1509949440UL,
                1543503872UL,
                1577058304UL,
                1342177280UL,
                1375731712UL,
                1409286144UL,
                1442840576UL,
                1476395008UL,
                1509949440UL,
                1543503872UL,
                1577058304UL,
                1610612736UL,
                1644167168UL,
                1677721600UL,
                1711276032UL,
                1744830464UL,
                1778384896UL,
                1811939328UL,
                1845493760UL,
                1610612736UL,
                1644167168UL,
                1677721600UL,
                1711276032UL,
                1744830464UL,
                1778384896UL,
                1811939328UL,
                1845493760UL,
                1879048192UL,
                1912602624UL,
                1946157056UL,
                1979711488UL,
                2013265920UL,
                2046820352UL,
                2080374784UL,
                2113929216UL,
                1879048192UL,
                1912602624UL,
                1946157056UL,
                1979711488UL,
                2013265920UL,
                2046820352UL,
                2080374784UL,
                2113929216UL,
                2147483648UL,
                2181038080UL,
                2214592512UL,
                2248146944UL,
                2281701376UL,
                2315255808UL,
                2348810240UL,
                2382364672UL,
                2147483648UL,
                2181038080UL,
                2214592512UL,
                2248146944UL,
                2281701376UL,
                2315255808UL,
                2348810240UL,
                2382364672UL,
                2415919104UL,
                2449473536UL,
                2483027968UL,
                2516582400UL,
                2550136832UL,
                2583691264UL,
                2617245696UL,
                2650800128UL,
                2415919104UL,
                2449473536UL,
                2483027968UL,
                2516582400UL,
                2550136832UL,
                2583691264UL,
                2617245696UL,
                2650800128UL,
                2684354560UL,
                2717908992UL,
                2751463424UL,
                2785017856UL,
                2818572288UL,
                2852126720UL,
                2885681152UL,
                2919235584UL,
                2684354560UL,
                2717908992UL,
                2751463424UL,
                2785017856UL,
                2818572288UL,
                2852126720UL,
                2885681152UL,
                2919235584UL,
                2952790016UL,
                2986344448UL,
                3019898880UL,
                3053453312UL,
                3087007744UL,
                3120562176UL,
                3154116608UL,
                3187671040UL,
                2952790016UL,
                2986344448UL,
                3019898880UL,
                3053453312UL,
                3087007744UL,
                3120562176UL,
                3154116608UL,
                3187671040UL,
                3221225472UL,
                3254779904UL,
                3288334336UL,
                3321888768UL,
                3355443200UL,
                3388997632UL,
                3422552064UL,
                3456106496UL,
                3221225472UL,
                3254779904UL,
                3288334336UL,
                3321888768UL,
                3355443200UL,
                3388997632UL,
                3422552064UL,
                3456106496UL,
                3489660928UL,
                3523215360UL,
                3556769792UL,
                3590324224UL,
                3623878656UL,
                3657433088UL,
                3690987520UL,
                3724541952UL,
                3489660928UL,
                3523215360UL,
                3556769792UL,
                3590324224UL,
                3623878656UL,
                3657433088UL,
                3690987520UL,
                3724541952UL,
                3758096384UL,
                3791650816UL,
                3825205248UL,
                3858759680UL,
                3892314112UL,
                3925868544UL,
                3959422976UL,
                3992977408UL,
                3758096384UL,
                3791650816UL,
                3825205248UL,
                3858759680UL,
                3892314112UL,
                3925868544UL,
                3959422976UL,
                3992977408UL,
                4026531840UL,
                4060086272UL,
                4093640704UL,
                4127195136UL,
                4160749568UL,
                4194304000UL,
                4227858432UL,
                4261412864UL,
                4026531840UL,
                4060086272UL,
                4093640704UL,
                4127195136UL,
                4160749568UL,
                4194304000UL,
                4227858432UL,
                4261412864UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                0UL,
                134217728UL,
                134217728UL,
                134217728UL,
                134217728UL,
                134217728UL,
                134217728UL,
                134217728UL,
                134217728UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                402653184UL,
                402653184UL,
                402653184UL,
                402653184UL,
                402653184UL,
                402653184UL,
                402653184UL,
                402653184UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                671088640UL,
                671088640UL,
                671088640UL,
                671088640UL,
                671088640UL,
                671088640UL,
                671088640UL,
                671088640UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                939524096UL,
                939524096UL,
                939524096UL,
                939524096UL,
                939524096UL,
                939524096UL,
                939524096UL,
                939524096UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4160749568UL
            },
            {
                0UL,
                67108864UL,
                0UL,
                67108864UL,
                0UL,
                67108864UL,
                0UL,
                67108864UL,
                134217728UL,
                201326592UL,
                134217728UL,
                201326592UL,
                134217728UL,
                201326592UL,
                134217728UL,
                201326592UL,
                268435456UL,
                335544320UL,
                268435456UL,
                335544320UL,
                268435456UL,
                335544320UL,
                268435456UL,
                335544320UL,
                402653184UL,
                469762048UL,
                402653184UL,
                469762048UL,
                402653184UL,
                469762048UL,
                402653184UL,
                469762048UL,
                536870912UL,
                603979776UL,
                536870912UL,
                603979776UL,
                536870912UL,
                603979776UL,
                536870912UL,
                603979776UL,
                671088640UL,
                738197504UL,
                671088640UL,
                738197504UL,
                671088640UL,
                738197504UL,
                671088640UL,
                738197504UL,
                805306368UL,
                872415232UL,
                805306368UL,
                872415232UL,
                805306368UL,
                872415232UL,
                805306368UL,
                872415232UL,
                939524096UL,
                1006632960UL,
                939524096UL,
                1006632960UL,
                939524096UL,
                1006632960UL,
                939524096UL,
                1006632960UL,
                1073741824UL,
                1140850688UL,
                1073741824UL,
                1140850688UL,
                1073741824UL,
                1140850688UL,
                1073741824UL,
                1140850688UL,
                1207959552UL,
                1275068416UL,
                1207959552UL,
                1275068416UL,
                1207959552UL,
                1275068416UL,
                1207959552UL,
                1275068416UL,
                1342177280UL,
                1409286144UL,
                1342177280UL,
                1409286144UL,
                1342177280UL,
                1409286144UL,
                1342177280UL,
                1409286144UL,
                1476395008UL,
                1543503872UL,
                1476395008UL,
                1543503872UL,
                1476395008UL,
                1543503872UL,
                1476395008UL,
                1543503872UL,
                1610612736UL,
                1677721600UL,
                1610612736UL,
                1677721600UL,
                1610612736UL,
                1677721600UL,
                1610612736UL,
                1677721600UL,
                1744830464UL,
                1811939328UL,
                1744830464UL,
                1811939328UL,
                1744830464UL,
                1811939328UL,
                1744830464UL,
                1811939328UL,
                1879048192UL,
                1946157056UL,
                1879048192UL,
                1946157056UL,
                1879048192UL,
                1946157056UL,
                1879048192UL,
                1946157056UL,
                2013265920UL,
                2080374784UL,
                2013265920UL,
                2080374784UL,
                2013265920UL,
                2080374784UL,
                2013265920UL,
                2080374784UL,
                2147483648UL,
                2214592512UL,
                2147483648UL,
                2214592512UL,
                2147483648UL,
                2214592512UL,
                2147483648UL,
                2214592512UL,
                2281701376UL,
                2348810240UL,
                2281701376UL,
                2348810240UL,
                2281701376UL,
                2348810240UL,
                2281701376UL,
                2348810240UL,
                2415919104UL,
                2483027968UL,
                2415919104UL,
                2483027968UL,
                2415919104UL,
                2483027968UL,
                2415919104UL,
                2483027968UL,
                2550136832UL,
                2617245696UL,
                2550136832UL,
                2617245696UL,
                2550136832UL,
                2617245696UL,
                2550136832UL,
                2617245696UL,
                2684354560UL,
                2751463424UL,
                2684354560UL,
                2751463424UL,
                2684354560UL,
                2751463424UL,
                2684354560UL,
                2751463424UL,
                2818572288UL,
                2885681152UL,
                2818572288UL,
                2885681152UL,
                2818572288UL,
                2885681152UL,
                2818572288UL,
                2885681152UL,
                2952790016UL,
                3019898880UL,
                2952790016UL,
                3019898880UL,
                2952790016UL,
                3019898880UL,
                2952790016UL,
                3019898880UL,
                3087007744UL,
                3154116608UL,
                3087007744UL,
                3154116608UL,
                3087007744UL,
                3154116608UL,
                3087007744UL,
                3154116608UL,
                3221225472UL,
                3288334336UL,
                3221225472UL,
                3288334336UL,
                3221225472UL,
                3288334336UL,
                3221225472UL,
                3288334336UL,
                3355443200UL,
                3422552064UL,
                3355443200UL,
                3422552064UL,
                3355443200UL,
                3422552064UL,
                3355443200UL,
                3422552064UL,
                3489660928UL,
                3556769792UL,
                3489660928UL,
                3556769792UL,
                3489660928UL,
                3556769792UL,
                3489660928UL,
                3556769792UL,
                3623878656UL,
                3690987520UL,
                3623878656UL,
                3690987520UL,
                3623878656UL,
                3690987520UL,
                3623878656UL,
                3690987520UL,
                3758096384UL,
                3825205248UL,
                3758096384UL,
                3825205248UL,
                3758096384UL,
                3825205248UL,
                3758096384UL,
                3825205248UL,
                3892314112UL,
                3959422976UL,
                3892314112UL,
                3959422976UL,
                3892314112UL,
                3959422976UL,
                3892314112UL,
                3959422976UL,
                4026531840UL,
                4093640704UL,
                4026531840UL,
                4093640704UL,
                4026531840UL,
                4093640704UL,
                4026531840UL,
                4093640704UL,
                4160749568UL,
                4227858432UL,
                4160749568UL,
                4227858432UL,
                4160749568UL,
                4227858432UL,
                4160749568UL,
                4227858432UL
            },
            {
                0UL,
                0UL,
                67108864UL,
                67108864UL,
                0UL,
                0UL,
                67108864UL,
                67108864UL,
                134217728UL,
                134217728UL,
                201326592UL,
                201326592UL,
                134217728UL,
                134217728UL,
                201326592UL,
                201326592UL,
                268435456UL,
                268435456UL,
                335544320UL,
                335544320UL,
                268435456UL,
                268435456UL,
                335544320UL,
                335544320UL,
                402653184UL,
                402653184UL,
                469762048UL,
                469762048UL,
                402653184UL,
                402653184UL,
                469762048UL,
                469762048UL,
                536870912UL,
                536870912UL,
                603979776UL,
                603979776UL,
                536870912UL,
                536870912UL,
                603979776UL,
                603979776UL,
                671088640UL,
                671088640UL,
                738197504UL,
                738197504UL,
                671088640UL,
                671088640UL,
                738197504UL,
                738197504UL,
                805306368UL,
                805306368UL,
                872415232UL,
                872415232UL,
                805306368UL,
                805306368UL,
                872415232UL,
                872415232UL,
                939524096UL,
                939524096UL,
                1006632960UL,
                1006632960UL,
                939524096UL,
                939524096UL,
                1006632960UL,
                1006632960UL,
                1073741824UL,
                1073741824UL,
                1140850688UL,
                1140850688UL,
                1073741824UL,
                1073741824UL,
                1140850688UL,
                1140850688UL,
                1207959552UL,
                1207959552UL,
                1275068416UL,
                1275068416UL,
                1207959552UL,
                1207959552UL,
                1275068416UL,
                1275068416UL,
                1342177280UL,
                1342177280UL,
                1409286144UL,
                1409286144UL,
                1342177280UL,
                1342177280UL,
                1409286144UL,
                1409286144UL,
                1476395008UL,
                1476395008UL,
                1543503872UL,
                1543503872UL,
                1476395008UL,
                1476395008UL,
                1543503872UL,
                1543503872UL,
                1610612736UL,
                1610612736UL,
                1677721600UL,
                1677721600UL,
                1610612736UL,
                1610612736UL,
                1677721600UL,
                1677721600UL,
                1744830464UL,
                1744830464UL,
                1811939328UL,
                1811939328UL,
                1744830464UL,
                1744830464UL,
                1811939328UL,
                1811939328UL,
                1879048192UL,
                1879048192UL,
                1946157056UL,
                1946157056UL,
                1879048192UL,
                1879048192UL,
                1946157056UL,
                1946157056UL,
                2013265920UL,
                2013265920UL,
                2080374784UL,
                2080374784UL,
                2013265920UL,
                2013265920UL,
                2080374784UL,
                2080374784UL,
                2147483648UL,
                2147483648UL,
                2214592512UL,
                2214592512UL,
                2147483648UL,
                2147483648UL,
                2214592512UL,
                2214592512UL,
                2281701376UL,
                2281701376UL,
                2348810240UL,
                2348810240UL,
                2281701376UL,
                2281701376UL,
                2348810240UL,
                2348810240UL,
                2415919104UL,
                2415919104UL,
                2483027968UL,
                2483027968UL,
                2415919104UL,
                2415919104UL,
                2483027968UL,
                2483027968UL,
                2550136832UL,
                2550136832UL,
                2617245696UL,
                2617245696UL,
                2550136832UL,
                2550136832UL,
                2617245696UL,
                2617245696UL,
                2684354560UL,
                2684354560UL,
                2751463424UL,
                2751463424UL,
                2684354560UL,
                2684354560UL,
                2751463424UL,
                2751463424UL,
                2818572288UL,
                2818572288UL,
                2885681152UL,
                2885681152UL,
                2818572288UL,
                2818572288UL,
                2885681152UL,
                2885681152UL,
                2952790016UL,
                2952790016UL,
                3019898880UL,
                3019898880UL,
                2952790016UL,
                2952790016UL,
                3019898880UL,
                3019898880UL,
                3087007744UL,
                3087007744UL,
                3154116608UL,
                3154116608UL,
                3087007744UL,
                3087007744UL,
                3154116608UL,
                3154116608UL,
                3221225472UL,
                3221225472UL,
                3288334336UL,
                3288334336UL,
                3221225472UL,
                3221225472UL,
                3288334336UL,
                3288334336UL,
                3355443200UL,
                3355443200UL,
                3422552064UL,
                3422552064UL,
                3355443200UL,
                3355443200UL,
                3422552064UL,
                3422552064UL,
                3489660928UL,
                3489660928UL,
                3556769792UL,
                3556769792UL,
                3489660928UL,
                3489660928UL,
                3556769792UL,
                3556769792UL,
                3623878656UL,
                3623878656UL,
                3690987520UL,
                3690987520UL,
                3623878656UL,
                3623878656UL,
                3690987520UL,
                3690987520UL,
                3758096384UL,
                3758096384UL,
                3825205248UL,
                3825205248UL,
                3758096384UL,
                3758096384UL,
                3825205248UL,
                3825205248UL,
                3892314112UL,
                3892314112UL,
                3959422976UL,
                3959422976UL,
                3892314112UL,
                3892314112UL,
                3959422976UL,
                3959422976UL,
                4026531840UL,
                4026531840UL,
                4093640704UL,
                4093640704UL,
                4026531840UL,
                4026531840UL,
                4093640704UL,
                4093640704UL,
                4160749568UL,
                4160749568UL,
                4227858432UL,
                4227858432UL,
                4160749568UL,
                4160749568UL,
                4227858432UL,
                4227858432UL
            },
            {
                0UL,
                33554432UL,
                67108864UL,
                100663296UL,
                0UL,
                33554432UL,
                67108864UL,
                100663296UL,
                134217728UL,
                167772160UL,
                201326592UL,
                234881024UL,
                134217728UL,
                167772160UL,
                201326592UL,
                234881024UL,
                268435456UL,
                301989888UL,
                335544320UL,
                369098752UL,
                268435456UL,
                301989888UL,
                335544320UL,
                369098752UL,
                402653184UL,
                436207616UL,
                469762048UL,
                503316480UL,
                402653184UL,
                436207616UL,
                469762048UL,
                503316480UL,
                536870912UL,
                570425344UL,
                603979776UL,
                637534208UL,
                536870912UL,
                570425344UL,
                603979776UL,
                637534208UL,
                671088640UL,
                704643072UL,
                738197504UL,
                771751936UL,
                671088640UL,
                704643072UL,
                738197504UL,
                771751936UL,
                805306368UL,
                838860800UL,
                872415232UL,
                905969664UL,
                805306368UL,
                838860800UL,
                872415232UL,
                905969664UL,
                939524096UL,
                973078528UL,
                1006632960UL,
                1040187392UL,
                939524096UL,
                973078528UL,
                1006632960UL,
                1040187392UL,
                1073741824UL,
                1107296256UL,
                1140850688UL,
                1174405120UL,
                1073741824UL,
                1107296256UL,
                1140850688UL,
                1174405120UL,
                1207959552UL,
                1241513984UL,
                1275068416UL,
                1308622848UL,
                1207959552UL,
                1241513984UL,
                1275068416UL,
                1308622848UL,
                1342177280UL,
                1375731712UL,
                1409286144UL,
                1442840576UL,
                1342177280UL,
                1375731712UL,
                1409286144UL,
                1442840576UL,
                1476395008UL,
                1509949440UL,
                1543503872UL,
                1577058304UL,
                1476395008UL,
                1509949440UL,
                1543503872UL,
                1577058304UL,
                1610612736UL,
                1644167168UL,
                1677721600UL,
                1711276032UL,
                1610612736UL,
                1644167168UL,
                1677721600UL,
                1711276032UL,
                1744830464UL,
                1778384896UL,
                1811939328UL,
                1845493760UL,
                1744830464UL,
                1778384896UL,
                1811939328UL,
                1845493760UL,
                1879048192UL,
                1912602624UL,
                1946157056UL,
                1979711488UL,
                1879048192UL,
                1912602624UL,
                1946157056UL,
                1979711488UL,
                2013265920UL,
                2046820352UL,
                2080374784UL,
                2113929216UL,
                2013265920UL,
                2046820352UL,
                2080374784UL,
                2113929216UL,
                2147483648UL,
                2181038080UL,
                2214592512UL,
                2248146944UL,
                2147483648UL,
                2181038080UL,
                2214592512UL,
                2248146944UL,
                2281701376UL,
                2315255808UL,
                2348810240UL,
                2382364672UL,
                2281701376UL,
                2315255808UL,
                2348810240UL,
                2382364672UL,
                2415919104UL,
                2449473536UL,
                2483027968UL,
                2516582400UL,
                2415919104UL,
                2449473536UL,
                2483027968UL,
                2516582400UL,
                2550136832UL,
                2583691264UL,
                2617245696UL,
                2650800128UL,
                2550136832UL,
                2583691264UL,
                2617245696UL,
                2650800128UL,
                2684354560UL,
                2717908992UL,
                2751463424UL,
                2785017856UL,
                2684354560UL,
                2717908992UL,
                2751463424UL,
                2785017856UL,
                2818572288UL,
                2852126720UL,
                2885681152UL,
                2919235584UL,
                2818572288UL,
                2852126720UL,
                2885681152UL,
                2919235584UL,
                2952790016UL,
                2986344448UL,
                3019898880UL,
                3053453312UL,
                2952790016UL,
                2986344448UL,
                3019898880UL,
                3053453312UL,
                3087007744UL,
                3120562176UL,
                3154116608UL,
                3187671040UL,
                3087007744UL,
                3120562176UL,
                3154116608UL,
                3187671040UL,
                3221225472UL,
                3254779904UL,
                3288334336UL,
                3321888768UL,
                3221225472UL,
                3254779904UL,
                3288334336UL,
                3321888768UL,
                3355443200UL,
                3388997632UL,
                3422552064UL,
                3456106496UL,
                3355443200UL,
                3388997632UL,
                3422552064UL,
                3456106496UL,
                3489660928UL,
                3523215360UL,
                3556769792UL,
                3590324224UL,
                3489660928UL,
                3523215360UL,
                3556769792UL,
                3590324224UL,
                3623878656UL,
                3657433088UL,
                3690987520UL,
                3724541952UL,
                3623878656UL,
                3657433088UL,
                3690987520UL,
                3724541952UL,
                3758096384UL,
                3791650816UL,
                3825205248UL,
                3858759680UL,
                3758096384UL,
                3791650816UL,
                3825205248UL,
                3858759680UL,
                3892314112UL,
                3925868544UL,
                3959422976UL,
                3992977408UL,
                3892314112UL,
                3925868544UL,
                3959422976UL,
                3992977408UL,
                4026531840UL,
                4060086272UL,
                4093640704UL,
                4127195136UL,
                4026531840UL,
                4060086272UL,
                4093640704UL,
                4127195136UL,
                4160749568UL,
                4194304000UL,
                4227858432UL,
                4261412864UL,
                4160749568UL,
                4194304000UL,
                4227858432UL,
                4261412864UL
            },
            {
                0UL,
                0UL,
                0UL,
                0UL,
                67108864UL,
                67108864UL,
                67108864UL,
                67108864UL,
                134217728UL,
                134217728UL,
                134217728UL,
                134217728UL,
                201326592UL,
                201326592UL,
                201326592UL,
                201326592UL,
                268435456UL,
                268435456UL,
                268435456UL,
                268435456UL,
                335544320UL,
                335544320UL,
                335544320UL,
                335544320UL,
                402653184UL,
                402653184UL,
                402653184UL,
                402653184UL,
                469762048UL,
                469762048UL,
                469762048UL,
                469762048UL,
                536870912UL,
                536870912UL,
                536870912UL,
                536870912UL,
                603979776UL,
                603979776UL,
                603979776UL,
                603979776UL,
                671088640UL,
                671088640UL,
                671088640UL,
                671088640UL,
                738197504UL,
                738197504UL,
                738197504UL,
                738197504UL,
                805306368UL,
                805306368UL,
                805306368UL,
                805306368UL,
                872415232UL,
                872415232UL,
                872415232UL,
                872415232UL,
                939524096UL,
                939524096UL,
                939524096UL,
                939524096UL,
                1006632960UL,
                1006632960UL,
                1006632960UL,
                1006632960UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1073741824UL,
                1140850688UL,
                1140850688UL,
                1140850688UL,
                1140850688UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1207959552UL,
                1275068416UL,
                1275068416UL,
                1275068416UL,
                1275068416UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1342177280UL,
                1409286144UL,
                1409286144UL,
                1409286144UL,
                1409286144UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1476395008UL,
                1543503872UL,
                1543503872UL,
                1543503872UL,
                1543503872UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1610612736UL,
                1677721600UL,
                1677721600UL,
                1677721600UL,
                1677721600UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1744830464UL,
                1811939328UL,
                1811939328UL,
                1811939328UL,
                1811939328UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1879048192UL,
                1946157056UL,
                1946157056UL,
                1946157056UL,
                1946157056UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2013265920UL,
                2080374784UL,
                2080374784UL,
                2080374784UL,
                2080374784UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2147483648UL,
                2214592512UL,
                2214592512UL,
                2214592512UL,
                2214592512UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2281701376UL,
                2348810240UL,
                2348810240UL,
                2348810240UL,
                2348810240UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2415919104UL,
                2483027968UL,
                2483027968UL,
                2483027968UL,
                2483027968UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2550136832UL,
                2617245696UL,
                2617245696UL,
                2617245696UL,
                2617245696UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2684354560UL,
                2751463424UL,
                2751463424UL,
                2751463424UL,
                2751463424UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2818572288UL,
                2885681152UL,
                2885681152UL,
                2885681152UL,
                2885681152UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                2952790016UL,
                3019898880UL,
                3019898880UL,
                3019898880UL,
                3019898880UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3087007744UL,
                3154116608UL,
                3154116608UL,
                3154116608UL,
                3154116608UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3221225472UL,
                3288334336UL,
                3288334336UL,
                3288334336UL,
                3288334336UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3355443200UL,
                3422552064UL,
                3422552064UL,
                3422552064UL,
                3422552064UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3489660928UL,
                3556769792UL,
                3556769792UL,
                3556769792UL,
                3556769792UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3623878656UL,
                3690987520UL,
                3690987520UL,
                3690987520UL,
                3690987520UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3758096384UL,
                3825205248UL,
                3825205248UL,
                3825205248UL,
                3825205248UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3892314112UL,
                3959422976UL,
                3959422976UL,
                3959422976UL,
                3959422976UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4026531840UL,
                4093640704UL,
                4093640704UL,
                4093640704UL,
                4093640704UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4160749568UL,
                4227858432UL,
                4227858432UL,
                4227858432UL,
                4227858432UL
            },
            {
                0UL,
                33554432UL,
                0UL,
                33554432UL,
                67108864UL,
                100663296UL,
                67108864UL,
                100663296UL,
                134217728UL,
                167772160UL,
                134217728UL,
                167772160UL,
                201326592UL,
                234881024UL,
                201326592UL,
                234881024UL,
                268435456UL,
                301989888UL,
                268435456UL,
                301989888UL,
                335544320UL,
                369098752UL,
                335544320UL,
                369098752UL,
                402653184UL,
                436207616UL,
                402653184UL,
                436207616UL,
                469762048UL,
                503316480UL,
                469762048UL,
                503316480UL,
                536870912UL,
                570425344UL,
                536870912UL,
                570425344UL,
                603979776UL,
                637534208UL,
                603979776UL,
                637534208UL,
                671088640UL,
                704643072UL,
                671088640UL,
                704643072UL,
                738197504UL,
                771751936UL,
                738197504UL,
                771751936UL,
                805306368UL,
                838860800UL,
                805306368UL,
                838860800UL,
                872415232UL,
                905969664UL,
                872415232UL,
                905969664UL,
                939524096UL,
                973078528UL,
                939524096UL,
                973078528UL,
                1006632960UL,
                1040187392UL,
                1006632960UL,
                1040187392UL,
                1073741824UL,
                1107296256UL,
                1073741824UL,
                1107296256UL,
                1140850688UL,
                1174405120UL,
                1140850688UL,
                1174405120UL,
                1207959552UL,
                1241513984UL,
                1207959552UL,
                1241513984UL,
                1275068416UL,
                1308622848UL,
                1275068416UL,
                1308622848UL,
                1342177280UL,
                1375731712UL,
                1342177280UL,
                1375731712UL,
                1409286144UL,
                1442840576UL,
                1409286144UL,
                1442840576UL,
                1476395008UL,
                1509949440UL,
                1476395008UL,
                1509949440UL,
                1543503872UL,
                1577058304UL,
                1543503872UL,
                1577058304UL,
                1610612736UL,
                1644167168UL,
                1610612736UL,
                1644167168UL,
                1677721600UL,
                1711276032UL,
                1677721600UL,
                1711276032UL,
                1744830464UL,
                1778384896UL,
                1744830464UL,
                1778384896UL,
                1811939328UL,
                1845493760UL,
                1811939328UL,
                1845493760UL,
                1879048192UL,
                1912602624UL,
                1879048192UL,
                1912602624UL,
                1946157056UL,
                1979711488UL,
                1946157056UL,
                1979711488UL,
                2013265920UL,
                2046820352UL,
                2013265920UL,
                2046820352UL,
                2080374784UL,
                2113929216UL,
                2080374784UL,
                2113929216UL,
                2147483648UL,
                2181038080UL,
                2147483648UL,
                2181038080UL,
                2214592512UL,
                2248146944UL,
                2214592512UL,
                2248146944UL,
                2281701376UL,
                2315255808UL,
                2281701376UL,
                2315255808UL,
                2348810240UL,
                2382364672UL,
                2348810240UL,
                2382364672UL,
                2415919104UL,
                2449473536UL,
                2415919104UL,
                2449473536UL,
                2483027968UL,
                2516582400UL,
                2483027968UL,
                2516582400UL,
                2550136832UL,
                2583691264UL,
                2550136832UL,
                2583691264UL,
                2617245696UL,
                2650800128UL,
                2617245696UL,
                2650800128UL,
                2684354560UL,
                2717908992UL,
                2684354560UL,
                2717908992UL,
                2751463424UL,
                2785017856UL,
                2751463424UL,
                2785017856UL,
                2818572288UL,
                2852126720UL,
                2818572288UL,
                2852126720UL,
                2885681152UL,
                2919235584UL,
                2885681152UL,
                2919235584UL,
                2952790016UL,
                2986344448UL,
                2952790016UL,
                2986344448UL,
                3019898880UL,
                3053453312UL,
                3019898880UL,
                3053453312UL,
                3087007744UL,
                3120562176UL,
                3087007744UL,
                3120562176UL,
                3154116608UL,
                3187671040UL,
                3154116608UL,
                3187671040UL,
                3221225472UL,
                3254779904UL,
                3221225472UL,
                3254779904UL,
                3288334336UL,
                3321888768UL,
                3288334336UL,
                3321888768UL,
                3355443200UL,
                3388997632UL,
                3355443200UL,
                3388997632UL,
                3422552064UL,
                3456106496UL,
                3422552064UL,
                3456106496UL,
                3489660928UL,
                3523215360UL,
                3489660928UL,
                3523215360UL,
                3556769792UL,
                3590324224UL,
                3556769792UL,
                3590324224UL,
                3623878656UL,
                3657433088UL,
                3623878656UL,
                3657433088UL,
                3690987520UL,
                3724541952UL,
                3690987520UL,
                3724541952UL,
                3758096384UL,
                3791650816UL,
                3758096384UL,
                3791650816UL,
                3825205248UL,
                3858759680UL,
                3825205248UL,
                3858759680UL,
                3892314112UL,
                3925868544UL,
                3892314112UL,
                3925868544UL,
                3959422976UL,
                3992977408UL,
                3959422976UL,
                3992977408UL,
                4026531840UL,
                4060086272UL,
                4026531840UL,
                4060086272UL,
                4093640704UL,
                4127195136UL,
                4093640704UL,
                4127195136UL,
                4160749568UL,
                4194304000UL,
                4160749568UL,
                4194304000UL,
                4227858432UL,
                4261412864UL,
                4227858432UL,
                4261412864UL
            },
            {
                0UL,
                0UL,
                33554432UL,
                33554432UL,
                67108864UL,
                67108864UL,
                100663296UL,
                100663296UL,
                134217728UL,
                134217728UL,
                167772160UL,
                167772160UL,
                201326592UL,
                201326592UL,
                234881024UL,
                234881024UL,
                268435456UL,
                268435456UL,
                301989888UL,
                301989888UL,
                335544320UL,
                335544320UL,
                369098752UL,
                369098752UL,
                402653184UL,
                402653184UL,
                436207616UL,
                436207616UL,
                469762048UL,
                469762048UL,
                503316480UL,
                503316480UL,
                536870912UL,
                536870912UL,
                570425344UL,
                570425344UL,
                603979776UL,
                603979776UL,
                637534208UL,
                637534208UL,
                671088640UL,
                671088640UL,
                704643072UL,
                704643072UL,
                738197504UL,
                738197504UL,
                771751936UL,
                771751936UL,
                805306368UL,
                805306368UL,
                838860800UL,
                838860800UL,
                872415232UL,
                872415232UL,
                905969664UL,
                905969664UL,
                939524096UL,
                939524096UL,
                973078528UL,
                973078528UL,
                1006632960UL,
                1006632960UL,
                1040187392UL,
                1040187392UL,
                1073741824UL,
                1073741824UL,
                1107296256UL,
                1107296256UL,
                1140850688UL,
                1140850688UL,
                1174405120UL,
                1174405120UL,
                1207959552UL,
                1207959552UL,
                1241513984UL,
                1241513984UL,
                1275068416UL,
                1275068416UL,
                1308622848UL,
                1308622848UL,
                1342177280UL,
                1342177280UL,
                1375731712UL,
                1375731712UL,
                1409286144UL,
                1409286144UL,
                1442840576UL,
                1442840576UL,
                1476395008UL,
                1476395008UL,
                1509949440UL,
                1509949440UL,
                1543503872UL,
                1543503872UL,
                1577058304UL,
                1577058304UL,
                1610612736UL,
                1610612736UL,
                1644167168UL,
                1644167168UL,
                1677721600UL,
                1677721600UL,
                1711276032UL,
                1711276032UL,
                1744830464UL,
                1744830464UL,
                1778384896UL,
                1778384896UL,
                1811939328UL,
                1811939328UL,
                1845493760UL,
                1845493760UL,
                1879048192UL,
                1879048192UL,
                1912602624UL,
                1912602624UL,
                1946157056UL,
                1946157056UL,
                1979711488UL,
                1979711488UL,
                2013265920UL,
                2013265920UL,
                2046820352UL,
                2046820352UL,
                2080374784UL,
                2080374784UL,
                2113929216UL,
                2113929216UL,
                2147483648UL,
                2147483648UL,
                2181038080UL,
                2181038080UL,
                2214592512UL,
                2214592512UL,
                2248146944UL,
                2248146944UL,
                2281701376UL,
                2281701376UL,
                2315255808UL,
                2315255808UL,
                2348810240UL,
                2348810240UL,
                2382364672UL,
                2382364672UL,
                2415919104UL,
                2415919104UL,
                2449473536UL,
                2449473536UL,
                2483027968UL,
                2483027968UL,
                2516582400UL,
                2516582400UL,
                2550136832UL,
                2550136832UL,
                2583691264UL,
                2583691264UL,
                2617245696UL,
                2617245696UL,
                2650800128UL,
                2650800128UL,
                2684354560UL,
                2684354560UL,
                2717908992UL,
                2717908992UL,
                2751463424UL,
                2751463424UL,
                2785017856UL,
                2785017856UL,
                2818572288UL,
                2818572288UL,
                2852126720UL,
                2852126720UL,
                2885681152UL,
                2885681152UL,
                2919235584UL,
                2919235584UL,
                2952790016UL,
                2952790016UL,
                2986344448UL,
                2986344448UL,
                3019898880UL,
                3019898880UL,
                3053453312UL,
                3053453312UL,
                3087007744UL,
                3087007744UL,
                3120562176UL,
                3120562176UL,
                3154116608UL,
                3154116608UL,
                3187671040UL,
                3187671040UL,
                3221225472UL,
                3221225472UL,
                3254779904UL,
                3254779904UL,
                3288334336UL,
                3288334336UL,
                3321888768UL,
                3321888768UL,
                3355443200UL,
                3355443200UL,
                3388997632UL,
                3388997632UL,
                3422552064UL,
                3422552064UL,
                3456106496UL,
                3456106496UL,
                3489660928UL,
                3489660928UL,
                3523215360UL,
                3523215360UL,
                3556769792UL,
                3556769792UL,
                3590324224UL,
                3590324224UL,
                3623878656UL,
                3623878656UL,
                3657433088UL,
                3657433088UL,
                3690987520UL,
                3690987520UL,
                3724541952UL,
                3724541952UL,
                3758096384UL,
                3758096384UL,
                3791650816UL,
                3791650816UL,
                3825205248UL,
                3825205248UL,
                3858759680UL,
                3858759680UL,
                3892314112UL,
                3892314112UL,
                3925868544UL,
                3925868544UL,
                3959422976UL,
                3959422976UL,
                3992977408UL,
                3992977408UL,
                4026531840UL,
                4026531840UL,
                4060086272UL,
                4060086272UL,
                4093640704UL,
                4093640704UL,
                4127195136UL,
                4127195136UL,
                4160749568UL,
                4160749568UL,
                4194304000UL,
                4194304000UL,
                4227858432UL,
                4227858432UL,
                4261412864UL,
                4261412864UL
            },
            {
                0UL,
                16777216UL,
                33554432UL,
                50331648UL,
                67108864UL,
                83886080UL,
                100663296UL,
                117440512UL,
                134217728UL,
                150994944UL,
                167772160UL,
                184549376UL,
                201326592UL,
                218103808UL,
                234881024UL,
                251658240UL,
                268435456UL,
                285212672UL,
                301989888UL,
                318767104UL,
                335544320UL,
                352321536UL,
                369098752UL,
                385875968UL,
                402653184UL,
                419430400UL,
                436207616UL,
                452984832UL,
                469762048UL,
                486539264UL,
                503316480UL,
                520093696UL,
                536870912UL,
                553648128UL,
                570425344UL,
                587202560UL,
                603979776UL,
                620756992UL,
                637534208UL,
                654311424UL,
                671088640UL,
                687865856UL,
                704643072UL,
                721420288UL,
                738197504UL,
                754974720UL,
                771751936UL,
                788529152UL,
                805306368UL,
                822083584UL,
                838860800UL,
                855638016UL,
                872415232UL,
                889192448UL,
                905969664UL,
                922746880UL,
                939524096UL,
                956301312UL,
                973078528UL,
                989855744UL,
                1006632960UL,
                1023410176UL,
                1040187392UL,
                1056964608UL,
                1073741824UL,
                1090519040UL,
                1107296256UL,
                1124073472UL,
                1140850688UL,
                1157627904UL,
                1174405120UL,
                1191182336UL,
                1207959552UL,
                1224736768UL,
                1241513984UL,
                1258291200UL,
                1275068416UL,
                1291845632UL,
                1308622848UL,
                1325400064UL,
                1342177280UL,
                1358954496UL,
                1375731712UL,
                1392508928UL,
                1409286144UL,
                1426063360UL,
                1442840576UL,
                1459617792UL,
                1476395008UL,
                1493172224UL,
                1509949440UL,
                1526726656UL,
                1543503872UL,
                1560281088UL,
                1577058304UL,
                1593835520UL,
                1610612736UL,
                1627389952UL,
                1644167168UL,
                1660944384UL,
                1677721600UL,
                1694498816UL,
                1711276032UL,
                1728053248UL,
                1744830464UL,
                1761607680UL,
                1778384896UL,
                1795162112UL,
                1811939328UL,
                1828716544UL,
                1845493760UL,
                1862270976UL,
                1879048192UL,
                1895825408UL,
                1912602624UL,
                1929379840UL,
                1946157056UL,
                1962934272UL,
                1979711488UL,
                1996488704UL,
                2013265920UL,
                2030043136UL,
                2046820352UL,
                2063597568UL,
                2080374784UL,
                2097152000UL,
                2113929216UL,
                2130706432UL,
                2147483648UL,
                2164260864UL,
                2181038080UL,
                2197815296UL,
                2214592512UL,
                2231369728UL,
                2248146944UL,
                2264924160UL,
                2281701376UL,
                2298478592UL,
                2315255808UL,
                2332033024UL,
                2348810240UL,
                2365587456UL,
                2382364672UL,
                2399141888UL,
                2415919104UL,
                2432696320UL,
                2449473536UL,
                2466250752UL,
                2483027968UL,
                2499805184UL,
                2516582400UL,
                2533359616UL,
                2550136832UL,
                2566914048UL,
                2583691264UL,
                2600468480UL,
                2617245696UL,
                2634022912UL,
                2650800128UL,
                2667577344UL,
                2684354560UL,
                2701131776UL,
                2717908992UL,
                2734686208UL,
                2751463424UL,
                2768240640UL,
                2785017856UL,
                2801795072UL,
                2818572288UL,
                2835349504UL,
                2852126720UL,
                2868903936UL,
                2885681152UL,
                2902458368UL,
                2919235584UL,
                2936012800UL,
                2952790016UL,
                2969567232UL,
                2986344448UL,
                3003121664UL,
                3019898880UL,
                3036676096UL,
                3053453312UL,
                3070230528UL,
                3087007744UL,
                3103784960UL,
                3120562176UL,
                3137339392UL,
                3154116608UL,
                3170893824UL,
                3187671040UL,
                3204448256UL,
                3221225472UL,
                3238002688UL,
                3254779904UL,
                3271557120UL,
                3288334336UL,
                3305111552UL,
                3321888768UL,
                3338665984UL,
                3355443200UL,
                3372220416UL,
                3388997632UL,
                3405774848UL,
                3422552064UL,
                3439329280UL,
                3456106496UL,
                3472883712UL,
                3489660928UL,
                3506438144UL,
                3523215360UL,
                3539992576UL,
                3556769792UL,
                3573547008UL,
                3590324224UL,
                3607101440UL,
                3623878656UL,
                3640655872UL,
                3657433088UL,
                3674210304UL,
                3690987520UL,
                3707764736UL,
                3724541952UL,
                3741319168UL,
                3758096384UL,
                3774873600UL,
                3791650816UL,
                3808428032UL,
                3825205248UL,
                3841982464UL,
                3858759680UL,
                3875536896UL,
                3892314112UL,
                3909091328UL,
                3925868544UL,
                3942645760UL,
                3959422976UL,
                3976200192UL,
                3992977408UL,
                4009754624UL,
                4026531840UL,
                4043309056UL,
                4060086272UL,
                4076863488UL,
                4093640704UL,
                4110417920UL,
                4127195136UL,
                4143972352UL,
                4160749568UL,
                4177526784UL,
                4194304000UL,
                4211081216UL,
                4227858432UL,
                4244635648UL,
                4261412864UL,
                4278190080UL
            }
        };


#endif
